/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: KEYMAP.H                                                   *
*                                                                      *
***********************************************************************/
#ifndef KEYMAP_H
#define KEYMAP_H


/*
**
** Imported "Include" Files
**
*/



/*
**
** Global Constant Definitions
**
*/
#define MAXEXTLEN 16



/*
**
** Global Enumeration Definitions
**
*/



/*
**
** Global Structure Definitions
**
*/
typedef struct
{
  int ulx;
  int uly;
  int lrx;
  int lry;
  int keycode;
}KeymapEntry;

typedef struct
{
  int keycode;
  char str[MAXEXTLEN];
}ExtEntry;

typedef struct
{
  int mapid;
  int highlight1;
  int highlight2;
  int highlight3;
  int highlight4;
  int width;
  int height;
  KeymapEntry *keys;
  unsigned short maxkeys;
  ExtEntry *exts;
  unsigned short maxexts;
}KeymapHandle;



/*
**
** Global Variable Declarations
**
*/



/*
**
**  NAME: keymapLoadMap()
**
** USAGE: KeymapHandle *keymapLoadMap(char *file);
**
** DESCR: This function will load the specified keymap.
**
** PARMS: The "file" parameter is a pointer to the buffer that contains the
**        path and filename.
**
** RETRN: If successful, a pointer to a KeymapHandle is returned. If an error
**        occurs during function exeuction, NULL is returned.
**
*/
KeymapHandle *keymapLoadMap(char *);



/*
**
**  NAME: keymapDeleteMap()
**
** USAGE: int keymapDeleteMap(KeymapHandle *handle);
**
** DESCR: This function will delete a KeymapHandle that was previously
**        created by a successful call to "keymapLoadHandle()".
**
** PARMS: The "handle" parameter is a pointer to a KeymapHandle.
**
** RETRN: If successful, '0' is returned. If an error occurs during function
**        execution, a non-zero value is returned that describes the error.
**
*/
int keymapDeleteMap(KeymapHandle *);

#endif /* KEYMAP_H */
