/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 *
 * Written by Jason Kingan and Greg Haerr
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/stat.h>

#include <unistd.h>

#include "apps.h"
#include "nanowm.h"
#include "catagories.h"
#include "settings.h"

#ifdef SYSTEM_MENUS
#include "sys_menu.h"
#endif

/* apps spacing for icon on the screentop*/

#define NXKEYBOARD	"nxkeyboard"
#define NXSCRIBBLE	"nxscribble"
#define DEFAULT_ICON	"nxdefapp.gif"

#define XOFFSET 20
#define YOFFSET 20
#define XSPACING (ICONWIDTH+20)
#define YSPACING (ICONHEIGHT+ICONTEXTHEIGHT+20)

GR_WINDOW_ID g_last_window;	/* last window created/opened */
extern GR_BOOL g_clear_window;

/* JHC 11/13/00 - I am adding the concept of virtual windows */
/* so apphead turns a array of pointers                      */

NXLISTHEAD      apphead;

APP * 		LastLaunchedApp = NULL;

static void apps_add_app_section(char *section);
/* static void apps_wndproc(win *window, GR_EVENT *ep); */

/* static void apps_child_handler(int id); */

int apps_load_info_from_file(char *filename, char *name, app_info_t *ainfo)
{
  int res;
  app_info_t tinfo;
  char buf[4096];

  res = IniGetString(name, NULL, "", buf, sizeof(buf), filename);
  
  if (!res || strlen(buf) == 0) return(0);
  
  bzero(&tinfo, sizeof(tinfo));
  
  IniGetString(name, "exec", "", tinfo.path, sizeof(tinfo.path), filename);
  IniGetString(name, "icon", "", tinfo.icon, sizeof(tinfo.icon), filename);
  IniGetString(name, "title", "", ainfo->title, sizeof(ainfo->title), filename);
  IniGetString(name, "args", "", tinfo.args, sizeof(tinfo.args), filename);
  IniGetString(name, "catagory", "", ainfo->catagory, sizeof(ainfo->catagory), filename);
  
  /* Now, expand out the paths if needed */
  desktop_expand_path(tinfo.path, ainfo->path, sizeof(ainfo->path));
  desktop_expand_path(tinfo.icon, ainfo->icon, sizeof(ainfo->icon));
  desktop_expand_path(tinfo.args, ainfo->args, sizeof(ainfo->args));
  
  return(1);
}

/* This function loads an application section, if it is found */

int apps_load_info(char *name, app_info_t *ainfo)
{
  char *filename;
  
  /* JHC 01/16/00 - See if the specified file has a config file locally */
  /* ie, in its exec path.  If so, try to load the info from there.  If */
  /* that tanks, then use the gobal config                              */

  if (ainfo->path)
    {
      int ret = 0;
      char pathname[256];
      char *p = strrchr(ainfo->path, '/');
      
      if (p)
	{
	  int size = (p - ainfo->path);

	  bzero(pathname, 256);

	  strncpy(pathname, ainfo->path, (int) size);

	  strcat(pathname, "/config.apps");
      
	  if (access(pathname, R_OK) == 0)
	    ret = apps_load_info_from_file(pathname, name, ainfo);
	  
	  if (ret) return(1);
	}
    }

  /* No go, get the info from the main config file */

  filename = nxGetConfigFile(NX_CONFIG_MAIN);
  
  if (!filename) return(0);
  return(apps_load_info_from_file(filename, name, ainfo));
}

/* nxSearchDir callback function*/
static int apps_search_cb(char *path, int mode, void *data)
{
  app_info_t ainfo;
  char *p = strrchr(path, '/');

  bzero(&ainfo, sizeof(ainfo));

  if (mode & NXDIR_EXEC)
    {
      /* kluge remove non-wanted programs for now*/
      if (!strcmp(p+1, "nxkbd") || !strcmp(p+1, "nxscribble") ||
	  !strcmp(p+1, "nxscrtop") || !strcmp(p+1, "nxkeyboard") )
	return 1;	/* skip, added manually*/
      
      strcpy(ainfo.path, path);
      apps_load_info(p + 1, &ainfo);
      return(apps_add_application(&ainfo));
    }
  
  return(0);
}

static void
apps_ini_cb(char *key, char *value, int data)
{
  /* bindir= add binaries from specified directory*/
  if (!strcmp(key, "bindir")) 
    {
      char epath[256];
      desktop_expand_path(value, epath, sizeof(epath));

      nxSearchDir(epath, NULL, NXDIR_FILE | NXDIR_EXEC, apps_search_cb, 0);
      return;
    }

  if (!strcmp(key, "app"))
    {
      apps_add_app_section(value);
      return;
    }
}
      
void load_applications(char *filename)
{  
  char	buf[4096];
  app_info_t ainfo;

  /* Set a signal handler for children that die */
  signal(SIGCHLD, application_handler);
  
  if (filename)
    {
      /* Read the desktop section looking for binary directories and applications */
    
      if (IniGetString("desktop", NULL, "", buf, sizeof(buf), filename))
	IniEnumKeyValues(buf, apps_ini_cb, 0);
    }
  
  /* Try to add the applications from the default directory */

  bzero(&ainfo, sizeof(ainfo));
  sprintf(ainfo.path, "%s/%s", get_desktop_config(DESKTOP_NXAPPDIR), NXKEYBOARD);
  ainfo.flags = (FL_KEYBOARD | FL_NOICON);
  apps_load_info(NXKEYBOARD, &ainfo);
  apps_add_application(&ainfo);

  bzero(&ainfo, sizeof(ainfo));
  sprintf(ainfo.path, "%s/%s", get_desktop_config(DESKTOP_NXAPPDIR), NXSCRIBBLE);
  ainfo.flags = FL_SCRIBBLE | FL_NOICON;
  apps_add_application(&ainfo);
}

/* add a new application from a config file section*/
static void
apps_add_app_section(char *section)
{
  app_info_t ainfo;

  bzero(&ainfo, sizeof(ainfo));

  if (!apps_load_info(section, &ainfo)) return;
  apps_add_application(&ainfo);
}

/* Helper app to parse out the args */

static int parse_args(char *name, char *argstr, char ***argv)
{
  char **argvptr = 0;

  char *h = argstr;
  char *t = 0;

  int argc = 1;
  int index = 0;

  /* If there are arguments, then count them */

  if (argstr)
    {
      while((t = strchr(h, ' ')))
	{ 
	  /* Skip to the next empty space */
	  for( ; *t == ' ' && *t != 0; t++);
	  
	  if (*t == 0)
	    break;

	  argc++;
	  h = t;
	}
    }
  
  /* Now, allocate the required room */
  *argv = (char **) malloc((argc + 1) * sizeof(char *));

  if (!*argv)
    return(0);
    
  argvptr = *argv;

  /* First step, add the program name */
  argvptr[0] = (char *) calloc(1, strlen(name) + 1);
  strcpy(argvptr[0], name);
  
  if (!argstr || !strlen(argstr))
    {
      argvptr[1] = (char *) 0;
      return(1);
    }

  /* Now add any args (if they exist) */

  h = argstr;
  
  while(index < argc)
    {
      int slen = 0;
      
      t = strchr(h, ' ');
      
      if (t)
	slen = (int) (t - h);
      else
	slen = strlen(argstr) - (int) (h - argstr);
      
      argvptr[++index] = (char *) calloc(1, slen + 1);
      
      /* If there was an error, skip to the next one */
      
      if (!argvptr[index])
	{
	  index--;
	
	  if (!t)
	    break;

	  /* Skip to the next word */
	  for( ; *t == ' ' && *t != 0; t++);
	  
	  if (*t == 0)
	    break;
	  
	  h = t;
	  continue;
	}
      
      if (t)
	strncpy(argvptr[index], h, slen);
      else
	strcpy(argvptr[index], h);
      
      if (!t)
	break;
      
      /* Skip to the next word */
      for( ; *t == ' ' && *t != 0; t++);
      
      if (*t == 0)
	break;
      
      h = t;
      continue;
    }

  /* Add a null pointer to the end */
  argvptr[++index] = (char *) 0;
  return(index);
}

  
/* add a new application to the desktop*/

int 
apps_add_application(app_info_t *app_info)
{
  APP *		app;
  char *	p;
  char		buf[256];
  
  struct stat     stat_t;
    
  if (!strlen(app_info->path)) return(1);
	
  /* check if the specified file even exists */

  if (stat(app_info->path, &stat_t) == -1)
    {
      printf("Error!  The application %s does not exist\n", app_info->path);
      return(1);
    }

  /* Check if the file is executable */
  /* FIXME:  Allow for associations here (.html, etc...) */

  if (!(stat_t.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)))
    {
      printf("Error!  The file %s is not a application\n", app_info->path);
      return(1);
    }

  app = nxItemNew(APP);
  if (!app) return 0;	/* stop enumeration*/

  /* set application flags*/
  app->m_flags = app_info->flags;
  
  /* Check for virtual window flags */
  app->m_virtual_flags = 0;
	
  if (app_info->flags & FL_STICKY) app->m_virtual_flags |= VW_STICKY;

  /* set executable path*/
  strzcpy(app->m_exec, app_info->path, sizeof(app->m_exec));
      
  /* Set the arguments */
  app->m_argc = parse_args(app->m_exec, app_info->args, &app->m_argv);
 
   if (app_info->flags & FL_NOICON)
    {
      /* Add this application to the main list */
      nxListAdd(&apphead, &app->link);
      
      /* Don't add it to a catagory, since it doesn't have a icon */
      return(1);
    }
   
   if (strlen(app_info->title))
     strzcpy(app->m_icontitle, app_info->title, sizeof(app->m_icontitle));
   else 
     {
      /* use basename if no title override*/
      p = strrchr(app_info->path, '/');
      strzcpy(app->m_icontitle, p? p+1: app_info->path,
	      sizeof(app->m_icontitle));
    }

  /* user passed icon, path.gif or default*/
  
  if (strlen(app_info->icon))
    app->m_icon_iid = GrLoadImageFromFile(app_info->icon, 0);
  else 
    {
      char fullpath[512];
      char localimagesdir[] = "images";
      char *searchdirs[2];
      
      searchdirs[0] = get_desktop_config(DESKTOP_IMAGEDIR);
      searchdirs[1] = localimagesdir;
      
      /* use basename.gif if no icon override*/
      p = strrchr(app_info->path, '/');
      sprintf(buf, "%s.gif", p? p+1: app_info->path);
      
      /* Try to load the icon */
      if (nxSearchForFile(buf, fullpath, sizeof(fullpath),
			  searchdirs, 2))
	app->m_icon_iid = GrLoadImageFromFile(fullpath, 0);
    }
  
  if (!app->m_icon_iid) 
    {
      char fullpath[512];
      
      /* If the regular icon wasn't found, then use the */
      /* default icon */
      
      sprintf(fullpath, "%s/%s", get_desktop_config(DESKTOP_IMAGEDIR),
	      DEFAULT_ICON);
      
      app->m_icon_iid = GrLoadImageFromFile(fullpath, 0);
      
      if (!app->m_icon_iid)
	{
	  free(app);
	  return 1;	/* don't display this file*/
	}
    }

  app->m_process_id = 0;
  app->m_container_wid = 0;
	

#ifdef NOTUSED

  /* create icon input window on desktop*/
  app->m_icon_wid = GrNewInputWindow(GR_ROOT_WINDOW_ID, x, y,
				     ICONWIDTH, 
				     ICONHEIGHT+ICONTEXTHEIGHT);
  add_window(app->m_icon_wid, GR_ROOT_WINDOW_ID, 0, apps_wndproc);
  
  GrSelectEvents(app->m_icon_wid, GR_EVENT_MASK_BUTTON_DOWN);
  
  GrMapWindow(app->m_icon_wid);
#endif

  app->m_icon_wid = create_icon_window();

  /* Add this application to the main list */
  nxListAdd(&apphead, &app->link);
  
  /* And then add it to the appropriate catagory */

  if (!strlen(app_info->catagory))
    add_app_to_catagory(0, app);
  else
    {
      int index = get_catagory_index(app_info->catagory);
      if (index != -1)
	add_app_to_catagory(index, app);
      else
	add_app_to_catagory(0, app);
    }
  
  /* set position for next icon*/

#ifdef NOTUSED
  if (!si.rows) GrGetScreenInfo(&si);
  
  if (x + XSPACING + XSPACING > si.ws_width) {
    x = XOFFSET;
    y += YSPACING;
  } else {
    x += XSPACING;
  }
#endif

  return 1;	/* ok*/
}

/* JHC 11/14 - Added this function to pull out the fork routine */
/* so it can be called outside of a buttondown */

#ifdef NOTUSED

void apps_start(APP *app)
{
  win *window;
  g_clear_window = 0;

  /* If it is already running, then bring it forward */

  if (app->m_process_id && app->m_container_wid) 
    {
      g_last_window = app->m_icon_wid;
      GrMapWindow(app->m_container_wid);
      window = find_window(app->m_container_wid);
      if (window)
	GrSetFocus(window->clientid);
      
#ifdef VIRTUAL_WINDOWS
      app->m_virtual_wid = get_current_vw();
      printf("Recording the virtual window to be %d\n", app->m_virtual_wid);

      /* virtswitch_redraw(); */
#endif
    } 
  else if (!(app->m_process_id = fork())) 
    {
      char path[256];
      char *p;

      /* Change to the working directory before executing */

      p = strrchr(app->m_exec, '/');
      strncpy(path, app->m_exec, (int) (p - app->m_exec));
      path[(int) (p - app->m_exec)] = 0;

      if (chdir(path))
	printf("Unable to switch to the working directory %s\n", path);

      /* Start the desired app */
      execvp(app->m_exec, app->m_argv);
      exit(0);
    } 
  else 
    {
      g_last_window = app->m_icon_wid;
      LastLaunchedApp = app;
    }
}
#endif

void 
apps_buttondown(GR_WINDOW_ID id)
{
  APP *app;
  
  app = find_app_container(id);
  if (!app)
    return;
  
  launch_application(app);
}

#ifdef NOTUSED

static void
apps_wndproc(win *window, GR_EVENT *ep)
{
	switch(ep->type) {
	case GR_EVENT_TYPE_BUTTON_DOWN:
		apps_buttondown(window->wid);
		break;
	}
}
#endif

#ifdef NOTUSED

static void 
apps_child_handler(int id)
{
	APP *	app;
	int 	status;
	int 	ret;

	/* if execlp() fails, we'll end up here before any new mapwindow...*/
	LastLaunchedApp = NULL;
	while (1) {
		ret = waitpid(-1, &status, WNOHANG);
		if (ret == -1 || ret == 0) {
			break;
		} else {
		  printf("process %d exit\n", ret);
		  app = find_app_pid(ret);
		  if (app) 
		    {
		      app->m_process_id = 0;
		      app->m_container_wid = 0;
		      
#ifdef VIRTUAL_WINDOWS
		      app->m_virtual_wid = 0;
#endif
		      
		      /* IF this is a one time thing, then get rid of it */
		      if (app->m_flags & FL_ONETIME)
			nxListRemove(&apphead, &app->link);
		    }
		  else printf("NOT FOUND\n");
		}
	}
}
#endif

#ifdef NOTUSED

/* 
 * This hides all of the windows for the given virtual window 
 */


void app_hide_windows(int virt)
{
  PNXLIST p;

  for (p=apphead.head; p; p=p->next) {
    APP *app = nxItemAddr(p, APP, link);
    
    if (app->m_container_wid && app->m_virtual_wid == virt)
      if (!(app->m_virtual_flags & VW_STICKY))  /* Check for stickyness */
	GrUnmapWindow(app->m_container_wid);
  }
}
  
/*
 * This will map all the active windows for a given virtual
 * window.  
 * FIXME:  We need a flag to indicate those apps that have been 
 * minimized or otherwise hidden 
 */

void app_show_windows(int virt, int focus)
{
  PNXLIST p;

  for (p=apphead.head; p; p=p->next) 
    {
      APP *app = nxItemAddr(p, APP, link);
      
      if (app->m_container_wid && app->m_virtual_wid == virt)
	{
	  if (!(app->m_virtual_flags & VW_STICKY))
	    GrMapWindow(app->m_container_wid);

	  /* If the new window has a stored focus, set it back again */
#ifdef NOTUSED
	  if (focus)
	    {
	      win *window = find_window(app->m_container_wid);
	      if (window)
		if (window->clientid == focus)
		  GrSetFocus(window->clientid);
	    }
#endif
	}
    }
}

#else /* VIRTUAL_WINDOWS */

void 
app_hide_windows()
{
  PNXLIST p;
  
  for (p=apphead.head; p; p=p->next) 
    {
      APP *app = nxItemAddr(p, APP, link);
      
      if (app->m_container_wid)
	  GrUnmapWindow(app->m_container_wid);
    }
}

#endif /* VIRTUAL_WINDOWS */

APP *
find_app_pid(int pid)
{
  PNXLIST	p = apphead.head;

  for (; p; p=p->next) {
    APP *app = nxItemAddr(p, APP, link);
    
    if (app->m_process_id == pid)
      return app;
  }
  return NULL;
}

APP *
find_app_path(char *path)
{
  PNXLIST	p = apphead.head;
  
  for (; p; p=p->next) {
    APP *app = nxItemAddr(p, APP, link);
    
    if (!strcmp(path, app->m_exec))
      return app;
  }
  return NULL;
}

APP *
find_app_flags(int flags)
{
  PNXLIST	p = apphead.head;
  
  for (; p; p=p->next) {
    APP *app = nxItemAddr(p, APP, link);
    
    if (app->m_flags & flags)
      return app;
  }
  return NULL;
}

APP *
find_app_container(GR_WINDOW_ID id)
{
  PNXLIST	p = apphead.head;

  for (; p; p=p->next) {
    APP *app = nxItemAddr(p, APP, link);
    
    if (app->m_icon_wid == id)
      return app;
  }
  return NULL;
  
}

/* JHC 11/14 - Added this to allow apps that aren't registered in the normal */
/* manner to run.  */

APP* apps_add_onetime(char *path)
{
  app_info_t ainfo;

  bzero(&ainfo, sizeof(ainfo));
  strcpy(ainfo.path, path);
  ainfo.flags = FL_NOICON | FL_ONETIME;

  if (!apps_add_application(&ainfo))
    return(0);
  
  return(find_app_path(path));
}

void apps_free_memory()
{
  /* Only called when we are going down.  */
  /* Free all the memory we have allocated */

  PNXLIST	p = apphead.head;
  PNXLIST       t;

  while(p)
    {
      APP *app = nxItemAddr(p, APP, link);
      t = p->next;

      nxListRemove(&apphead, &app->link);

      if (app->m_argc && app->m_argv)
	{
	  int i;
	  for(i = 0; i < app->m_argc; i++)
	    if (app->m_argv[i])
	      free(app->m_argv[i]);

	  free(app->m_argv);
	}

      free(app);

      p = t;
    }
}

#ifdef SYSTEM_MENUS
int create_apps_menu(wm_menu_t **data)
{
  int i, index = 0;
  int count = get_catagory_count();
  wm_menu_t *ptr;

  count++;
  *data = create_system_menu(count);

  if (!*data) return(0);
  else ptr = *data;

  for(i = 0; i < count; i++)
    {
      char name[25];

      wm_menu_t *cmenu = create_catagory_menu(i, name);
      
      if (cmenu) ADD_SYS_MENU_CMENU(&ptr[index++], name, cmenu);
    }

  ADD_SYS_MENU_END(&ptr[index++]); 
  return(index);
}

#endif
  
