/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 *
 * Written by Jason Kingan and Greg Haerr
 */
#include "nxlib.h"

#define ICONWIDTH	48
#define ICONHEIGHT	48
#define ICONTEXTHEIGHT	13

#define COLOR_TASKBAR 		GrGetSysColor(GR_COLOR_3DLIGHT)
#define COLOR_TASKBARHILIGHT	GrGetSysColor(GR_COLOR_BTNHIGHLIGHT)
#define COLOR_ICONTEXT		GrGetSysColor(GR_COLOR_ACTIVECAPTIONTEXT)
#define COLOR_DESKTOP 		GrGetSysColor(GR_COLOR_DESKTOP)
#define COLOR_REALDESKTOP 	GR_RGB(1,1,1)

void root_create(void);
void load_applications(char *);
void apps_buttondown(GR_WINDOW_ID id);
void home_create(void);
void date_create(void);
void backlight_create(void);
void scribble_create(void);
void keyboard_create(void);
void battery_create(void);

#ifdef VIRTUAL_WINDOWS
void cat_switch_create(void);
#endif

#define FL_KEYBOARD	0x001	/* keyboard app*/
#define FL_SCRIBBLE	0x002	/* scribble app*/
#define FL_NOICON	0x004	/* don't create icon or icon window*/
#define FL_ONETIME      0x008   /* Only run once */
#ifdef VIRTUAL_WINDOWS
#define FL_STICKY       0x010
#endif

#ifdef VIRTUAL_WINDOWS
#define VW_STICKY       0x01
#endif


/* desktop and taskbar applications kept here*/
typedef struct {
	NXLIST	link;
	char m_exec[64];	/* executable filename*/
	char m_icontitle[64];	/* displayed icon title*/  
        int  m_argc;
        char **m_argv;
	int m_flags;		/* application flags*/
	int m_icon_wid;		/* icon input window id */
	int m_icon_iid;		/* icon image id */
	int m_container_wid;	/* container window id*/
	int m_process_id;	/* linux process id of running app*/
#ifdef VIRTUAL_WINDOWS
  int m_virtual_wid;            /* The current virtual window that this app is associated with */
  int m_virtual_flags;          /* Virtual window flags */
#endif

} APP;

typedef struct
{
  NXLIST next;
  int pid;
  APP *app;
} PROCESS;
  
typedef struct
{
  char path[256];
  char icon[256];
  char title[128];
  char args[256];
  char catagory[25];
  int  flags;
} app_info_t;

extern NXLISTHEAD apphead;

extern APP * LastLaunchedApp;
extern GR_WINDOW_ID g_last_window;

void app_hide_windows();
APP *find_app_flags(int flags);
APP *find_app_path(char *path); /* Find a app based on the path */
APP *find_app_pid(int pid);
APP *find_app_container(GR_WINDOW_ID id);

int apps_add_application(app_info_t *app_info);

void application_handler(int id);
int launch_application(APP *app);

void app_show_windows(int, int);

APP *apps_add_onetime(char *path); /* Add a onetime app to run */
