/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 *
 * Written by Jason Kingan and Greg Haerr
 */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "nano-X.h"
#include "nxdraw.h"
#include "apps.h"
#include "./h3600_ts.h"
#include "nanowm.h"

static GR_IMAGE_ID idON;
static GR_IMAGE_ID idOFF;
static GR_BOOL	   blON = 1;

static void backlight_wndproc(win *window, GR_EVENT *ep);
static void set_backlight(int value);


#define BLOFFICON "bloff.gif"
#define BLONICON  "blon.gif"

void 
backlight_create(void)
{
	GR_WINDOW_ID	wid;
	GR_SCREEN_INFO	si;

	GrGetScreenInfo(&si);
	wid = GrNewWindowEx(GR_WM_PROPS_NODECORATE, NULL, GR_ROOT_WINDOW_ID,
		2, si.ws_height + 2, 20, 20, COLOR_TASKBAR);
	add_window(wid, GR_ROOT_WINDOW_ID, 0, backlight_wndproc);

	GrSelectEvents(wid, GR_EVENT_MASK_BUTTON_DOWN | GR_EVENT_MASK_EXPOSURE);
	GrMapWindow(wid);

}

static void 
backlight_exposure(win *window, GR_EVENT_EXPOSURE *ep)
{  
  GR_GC_ID gc = GrNewGC();
  
  if (!idON || !idOFF)
    {
      char path[256];
    
      if (!idON)
	{
	  sprintf(path, "%s/%s", 
		  get_desktop_config(DESKTOP_IMAGEDIR),
		  BLONICON);
	  idON = GrLoadImageFromFile(path, 0);
	}
      
      if (!idOFF)
	{
	  sprintf(path, "%s/%s", 
		  get_desktop_config(DESKTOP_IMAGEDIR),
		  BLOFFICON);
	  idOFF = GrLoadImageFromFile(path, 0);
	}
    }

  /* draw image in actual size*/
  GrDrawImageToFit(ep->wid, gc, 0, 0, -1, -1, blON? idON: idOFF);
  
  GrDestroyGC(gc);
}

static void 
backlight_buttondown(win *window, GR_EVENT_BUTTON *ep)
{
	GR_EVENT_EXPOSURE 	expose;
	GR_WINDOW_INFO		info;

	blON = !blON;

	/* dummy full area expose event*/
	GrGetWindowInfo(ep->wid, &info);
	expose.type = GR_EVENT_TYPE_EXPOSURE;
	expose.wid = ep->wid;
	expose.x = 0;
	expose.y = 0;
	expose.width = info.width;
	expose.height = info.height;
	backlight_exposure(window, &expose);
	set_backlight(blON);
}

static void
backlight_wndproc(win *window, GR_EVENT *ep)
{
	switch(ep->type) {
	case GR_EVENT_TYPE_EXPOSURE:
		backlight_exposure(window, &ep->exposure);
		break;
	case GR_EVENT_TYPE_BUTTON_DOWN:
		backlight_buttondown(window, &ep->button);
		break;
	}
}

static void 
set_backlight(int value)
{
	int 		fd;
	FLITE_IN 	bl;

	fd = open("/dev/h3600_ts", O_RDWR);
	if (fd == -1)
		return;

	bl.mode = 2;
	bl.pwr = value;
	bl.brightness = 255;
	ioctl(fd, FLITE_ON, (void *) &bl);

	close(fd);
}
