/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 *
 * Written by Jason Kingan and Greg Haerr
 */
#include <stdio.h>
#include <time.h>
#include "nano-X.h"
#include "nanowm.h"
#include "apps.h"

#ifdef SYSTEM_MENUS
#include "sys_menu.h"
#endif

#define DATE_12_HOUR_MODE 0x00
#define DATE_24_HOUR_MODE 0x01

static int DateToggle = 0;
static int DateMode = DATE_12_HOUR_MODE;

static char *DAY[] = {
	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
};

static char *MON[] = {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

static void date_wndproc(win *window, GR_EVENT *ep);

void 
date_create(void)
{
	win		*window;
	GR_WINDOW_ID	wid;
	GR_SCREEN_INFO	si;

	GrGetScreenInfo(&si);
	wid = GrNewWindowEx(GR_WM_PROPS_NODECORATE, NULL, GR_ROOT_WINDOW_ID,
	     32, si.ws_height + 4, 70, 16, COLOR_TASKBAR);
	window = add_window(wid, GR_ROOT_WINDOW_ID, 0, date_wndproc);

	GrSelectEvents(wid, GR_EVENT_MASK_BUTTON_DOWN | GR_EVENT_MASK_BUTTON_UP | 
		       GR_EVENT_MASK_EXPOSURE);
	GrMapWindow(wid);

	timeout_setproc(date_wndproc, window);
}

static void 
date_exposure(win *window, GR_EVENT_EXPOSURE *ep)
{
	time_t t;
	struct tm *tv;
	char buf[20];
	GR_GC_ID gc = GrNewGC();
	GR_FONT_ID fontid = GrCreateFont(GR_FONT_GUI_VAR, 0, NULL);

	GrSetGCForeground(gc, COLOR_TASKBAR);
	GrSetGCBackground(gc, COLOR_DESKTOP);
	GrFillRect(ep->wid, gc, ep->x, ep->y, ep->width, ep->height);

	t = time(0);
	tv = localtime(&t);

	if (DateToggle) {
		sprintf(buf, "%s %s %d", DAY[tv->tm_wday], MON[tv->tm_mon],
			tv->tm_mday);
	} 
	else 
	  {
	    if (DateMode == DATE_24_HOUR_MODE)
	      {
		sprintf(buf, "%02d:%02d:%02d   ",
			tv->tm_hour, tv->tm_min, tv->tm_sec);
	      }
	    else
	      {
		if (tv->tm_hour > 11)
		  sprintf(buf, "%d:%02d:%02d pm",
			  (tv->tm_hour > 12 ?  tv->tm_hour - 12 : tv->tm_hour),
			  tv->tm_min, tv->tm_sec);
		else
		  sprintf(buf, "%d:%02d:%02d am",
			  (tv->tm_hour == 0? 12: tv->tm_hour),
			  tv->tm_min, tv->tm_sec);
	      }
	  }
	
	GrSetGCForeground(gc, COLOR_DESKTOP);
	GrSetGCBackground(gc, COLOR_TASKBAR);
	GrSetGCFont(gc, fontid);
	GrText(ep->wid, gc, 0, 1, buf, -1, GR_TFTOP);

	GrDestroyFont(fontid);
	GrDestroyGC(gc);
}

static void
date_redraw(win *window)
{
	GR_EVENT_EXPOSURE 	expose;
	GR_WINDOW_INFO		info;

	/* dummy full area expose event*/
	GrGetWindowInfo(window->wid, &info);
	expose.type = GR_EVENT_TYPE_EXPOSURE;
	expose.wid = window->wid;
	expose.x = 0;
	expose.y = 0;
	expose.width = info.width;
	expose.height = info.height;
	date_exposure(window, &expose);
}

#ifdef SYSTEM_MENUS

static void date_set_format(void *val)
{
  if ((int) val == DATE_24_HOUR_MODE)
    DateMode = DATE_24_HOUR_MODE;
  else
    DateMode = DATE_12_HOUR_MODE;
}
 
static wm_menu_t date_sys_menu[] = {
{ "12 Hour", MENU_TYPE_CHECK, date_set_format, (void *) DATE_12_HOUR_MODE},
{ "24 Hour", MENU_TYPE_CHECK, date_set_format, (void *) DATE_24_HOUR_MODE},
{ "", MENU_TYPE_END, 0}};

#endif

static void 
date_buttondown(win *window, GR_EVENT_BUTTON *ep)
{
#ifdef SYSTEM_MENUS
  if (ep->buttons & GR_BUTTON_R)
    {
      GR_SCREEN_INFO si;
      GrGetScreenInfo(&si);
      
      date_sys_menu[0].checked = GR_FALSE;
      date_sys_menu[1].checked = GR_FALSE;
      
      if (DateMode == DATE_12_HOUR_MODE)
	date_sys_menu[0].checked = GR_TRUE;
      else
	date_sys_menu[1].checked = GR_TRUE;
      
      if (open_system_menu(GR_ROOT_WINDOW_ID, 32, si.ws_height, 
			    MENU_DIR_UP, date_sys_menu))

      return;
    }
#endif

  DateToggle = !DateToggle;
  date_redraw(window);
}

static void
date_wndproc(win *window, GR_EVENT *ep)
{
	switch(ep->type) {
	case GR_EVENT_TYPE_EXPOSURE:
		date_exposure(window, &ep->exposure);
		break;
	case GR_EVENT_TYPE_BUTTON_DOWN:
		date_buttondown(window, &ep->button);
		break;
	case GR_EVENT_TYPE_TIMEOUT:
		date_redraw(window);
		break;
	}
}
