#include <stdio.h>

#include "apps.h"
#include "nanowm.h"
#include "catagories.h"
#include "settings.h"

#define XOFFSET 20
#define YOFFSET 20
#define XSPACING (ICONWIDTH+20)
#define YSPACING (ICONHEIGHT+ICONTEXTHEIGHT+20)

void hide_icon_list(void)
{
  CATAGORY *cat = get_current_catagory();
  PNXLIST	p;
  
  if (!cat)
    return;

  p = cat->applist.head;

  while(p)
    {
      CATAPPLIST *catlist = nxItemAddr(p, CATAPPLIST, link);
      APP *app = catlist->app;

      GrUnmapWindow(app->m_icon_wid);
      
      p = p->next;
    }
}
     
void draw_current_iconlist(void)
{
  PNXLIST	p;
  int xpos = XOFFSET;
  int ypos = YOFFSET;
  int width, height, baseline, x;

  GR_GC_ID gc = GrNewGC();
  GR_FONT_ID	fnt = GrCreateFont(GR_FONT_GUI_VAR, 0, NULL);
  GR_SCREEN_INFO si;

  CATAGORY *cat = get_current_catagory();

  if (!cat) 
    {
      printf("No catagory found!\n");
      return;
    }
  
  GrGetScreenInfo(&si);

  /* draw icons*/
  GrSetGCForeground(gc, wm_get_color(WM_ICONTEXT));
  GrSetGCBackground(gc, wm_get_color(WM_ICONCOLOR));
  GrSetGCFont(gc, fnt);

  p = cat->applist.head;

  for (; p; p=p->next) 
    {
      CATAPPLIST *catlist = nxItemAddr(p, CATAPPLIST, link);
      APP *app = catlist->app;
  
      if (!app->m_icon_iid) continue;
      if (app->m_flags & (FL_SCRIBBLE | FL_KEYBOARD | FL_NOICON))
	continue;

      GrMoveWindow(app->m_icon_wid, xpos, ypos);
      GrMapWindow(app->m_icon_wid);
      
      GrDrawImageToFit(GR_ROOT_WINDOW_ID, gc,
		       xpos, ypos, -1, -1, app->m_icon_iid);
      
      /* center text under icon*/
      GrGetGCTextSize(gc, app->m_icontitle, -1,
		      GR_TFASCII|GR_TFTOP, &width, &height,&baseline);
      
      x = (ICONWIDTH - width)/2;
      if (x < 0)
	x /= 2;	/* try to center under icon*/
      x += xpos;
      GrText(GR_ROOT_WINDOW_ID, gc, x, ypos+ICONHEIGHT,
	     app->m_icontitle, -1, GR_TFASCII|GR_TFTOP);
   
  
      if (xpos + XSPACING + XSPACING > si.ws_width)
	{
	  xpos = 20;
	  ypos += YSPACING;
	}
      else
	{
	  xpos += XSPACING;
	}
      
    }

  GrDestroyFont(fnt);
  GrDestroyGC(gc);
}
  
static void icon_handler(win *window, GR_EVENT *ep)
{
  switch(ep->type) 
    {
    case GR_EVENT_TYPE_BUTTON_DOWN:
      apps_buttondown(window->wid);

      break;
    }
}

GR_WINDOW_ID create_icon_window()
{
  GR_WINDOW_ID wid = GrNewInputWindow(GR_ROOT_WINDOW_ID, 20, 20,
				      ICONWIDTH, ICONHEIGHT + ICONTEXTHEIGHT);
 
  add_window(wid, GR_ROOT_WINDOW_ID, 0, icon_handler);

  GrSelectEvents(wid, 
		 GR_EVENT_MASK_BUTTON_DOWN | GR_EVENT_MASK_BUTTON_UP);
  
  return(wid);
}
