/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 * Copyright (C) 2000 Greg Haerr <greg@censoft.com>
 * Copyright (C) 2000 Alex Holden <alex@linuxhacker.org>
 */
#include <stdio.h>
#include <signal.h>
#include "nano-X.h"
/* Uncomment this if you want debugging output from this file */
/* #define DEBUG */
#include "nanowm.h"
#include "apps.h"
#include "catagories.h"

#ifdef SYSTEM_MENUS
#include "sys_menu.h"
#endif

#define CONFIG_FILE	"config.nxscrtop"

/* Jordan Crouse 11/01 - Added this so more than one root app */
/* can have access to the timer */

static struct timeout_proc_t
{
  wndproc proc;
  win *window;
} timeout_procs[3];
static int timeout_procs_count = 0;

static void mainloop(void);

/*
 * This the universal, get-outta here function that closes everything down and bails.  
 */

void close_nxscrtop(int id)
{
  win *window;

  printf("We have been killed.  Responsibily freeing resources....\n");
  
  window = find_window(GR_ROOT_WINDOW_ID);
  remove_window_and_children(window);

  apps_free_memory();
  root_free_memory();
  catagory_free_memory();
  cat_switch_free_memory();

  nxFreeConfigFiles();

  exit(0);
}

int
main(int argc, char *argv[])
{
  /* Register to get alerted when we get a SIGKILL, so we can free any allocated memory */

  signal(SIGINT, close_nxscrtop);
  signal(SIGTERM,close_nxscrtop);
  signal(SIGHUP, close_nxscrtop);

  if(GrOpen() < 0) 
    {
      fprintf(stderr, "Couldn't connect to Nano-X server!\n");
      exit(1);
    }
  
  /* Search for the global configuration file and store the location */	
  nxLoadConfigFile(CONFIG_FILE, NX_CONFIG_MAIN);

  /* Now load all of the desktop settings (paths, colors, etc) */
  load_desktop_paths(nxGetConfigFile(NX_CONFIG_MAIN));
	
  /* Next up, load all of the catagory information */
  load_catagory_config(nxGetConfigFile(NX_CONFIG_MAIN));

  /* Now, search for all of the applications for our icon lists */
  load_applications(nxGetConfigFile(NX_CONFIG_MAIN));

  GrReqShmCmds(65536L);
  
  /* pass errors through main loop, don't exit*/
  GrSetErrorHandler(NULL);
  
  root_create();
  mainloop();
  return 0;
}

static void
mainloop(void)
{
	GR_EVENT event;
	win *	window;
	int i;

	while(1) { 
		GrGetNextEventTimeout(&event, 500);

		switch(event.type) {
		/* all these events share their wid member location*/
		case GR_EVENT_TYPE_EXPOSURE:
		case GR_EVENT_TYPE_BUTTON_DOWN:
		case GR_EVENT_TYPE_BUTTON_UP:
		case GR_EVENT_TYPE_MOUSE_POSITION:
		case GR_EVENT_TYPE_MOUSE_ENTER:
		case GR_EVENT_TYPE_MOUSE_EXIT:
		case GR_EVENT_TYPE_FOCUS_IN:
		case GR_EVENT_TYPE_FOCUS_OUT:
		case GR_EVENT_TYPE_KEY_DOWN:
		case GR_EVENT_TYPE_KEY_UP:
		case GR_EVENT_TYPE_UPDATE:
			window = find_window(((GR_EVENT_GENERAL *)&event)->wid);

#ifdef SYSTEM_MENUS
		  /* If a system menu is active, send it everything */
		  /* if it returns zero, then process the click as normal */
		  
		  if (system_menu_active() == GR_TRUE)
		    {
		      if (handle_system_menu(&event) == 1)
			break;
		    }
#endif
		  
		  if (window && window->proc)
		    window->proc(window, &event);
		  break;

		case GR_EVENT_TYPE_CHLD_UPDATE:
			window = find_window(
				((GR_EVENT_UPDATE *)&event)->subwid);

			/* 
			 * Add unknown newly mapped window to window manager
			 * internal data structures.  This will also create
			 * a new parent container window.
			 */
			if (!window && event.update.utype == GR_UPDATE_MAP) {
				client_updatemap_new(
					((GR_EVENT_UPDATE *)&event)->subwid);
			}

			if (window && window->proc)
				window->proc(window, &event);
			break;

		case GR_EVENT_TYPE_TIMEOUT:
			for(i = 0; i < timeout_procs_count; i++)
				timeout_procs[i].proc(timeout_procs[i].window,
					&event);
			break;


		case GR_EVENT_TYPE_ERROR:
		  	/* 
			 * Don't call default error handler on errors,
			 * this would cause the window manager to
			 * exit on the GrClose() call, on behalf
			 * of the client!
			 */
			printf("nxscrtop: Error (%s) ", event.error.name);
			printf(nxErrorStrings[event.error.code],
				event.error.id);
			break;

		default:
			printf("Got unexpected event %d\n", event.type);
			break;
		}
	}
}

/* temp kluge for timeout handling*/
void
timeout_setproc(wndproc proc, win *window)
{
	timeout_procs[timeout_procs_count].proc = proc;
	timeout_procs[timeout_procs_count++].window = window;
}
