/* 
 * Nano-X Application Framework Library
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 * Copyright (c) 2000 Greg Haerr <greg@censoft.com>
 */
#include "nano-X.h"

/* PDA style, thin border, maximized, caption/closebox*/
#define STYLE_PDA		(/*GR_WM_PROPS_BORDER |*/ \
				GR_WM_PROPS_MAXIMIZE |\
				GR_WM_PROPS_CAPTION |\
				GR_WM_PROPS_NOMOVE |\
				GR_WM_PROPS_CLOSEBOX)

/* Webpad style, thick border, caption/closebox*/
#define STYLE_WEBPAD		(GR_WM_PROPS_APPFRAME | \
				GR_WM_PROPS_CAPTION |\
				GR_WM_PROPS_CLOSEBOX)

/* nxgeom.c - geometry parser*/
int		nxGetGeometry(GR_CHAR *user_geom, GR_CHAR *def_geom,
			GR_WM_PROPS style, GR_COORD *x_return,
			GR_COORD *y_return, GR_SIZE *width_return,
			GR_SIZE *height_return);

/* nxutil.c - utility routines*/

/* return default window decoration style*/
GR_WM_PROPS	nxGetDefaultWindowStyle(void);

/* calculate container size from client style/size*/
void		nxCalcNCSize(GR_WM_PROPS style, GR_SIZE wClient,
			GR_SIZE hClient, GR_COORD *xCliOffset,
			GR_COORD *yCliOffset, GR_SIZE *wContainer,
			GR_SIZE *hContainer);

/* calculate client size from container style/size*/
void		nxCalcClientSize(GR_WM_PROPS style, GR_SIZE wContainer,
			GR_SIZE hContainer, GR_COORD *xCliOffset,
			GR_COORD *yCliOffset, GR_SIZE *wClient,
			GR_SIZE *hClient);

/* directory search defines*/

/* enumeration function is called until 0 value returned*/
typedef int (*nxEnumProc)(char *path, int mode, void *);

#define NXDIR_FILE	01	/* regular files*/
#define NXDIR_EXEC	02	/* executable files*/

/* search passed directory for files according to flag options*/
int nxSearchDir(char *dir, char *pattern, int flags, nxEnumProc cbProc, void *data);

/* application initialization defines for use with nxCreateAppWindow*/
typedef struct {
	int	type;		/* init type*/
	char *	optname;	/* option name*/
	char *	defvalue;	/* default value*/
} nxARGS;

GR_WINDOW_ID	nxCreateAppWindow(int *aac, char ***aav, nxARGS *alist);

/* command line options*/
#define nxstrTITLE		"-title"
#define nxstrGEOMETRY		"-geom"
#define nxstrBACKGROUND		"-background"
#define nxstrSTYLE		"-style"

#define nxtypeTITLE		1
#define nxtypeGEOMETRY		2
#define nxtypeBACKGROUND	3
#define nxtypeSTYLE		4
#define nxSTR			0x0000
#define nxINT			0x0100

#define nxTITLE(title)	    {nxtypeTITLE|nxSTR,nxstrTITLE,title}
#define nxGEOMETRY(geom)    {nxtypeGEOMETRY|nxINT,nxstrGEOMETRY,(char *)(geom)}
#define nxBACKGROUND(color) {nxtypeBACKGROUND|nxINT,nxstrBACKGROUND,\
				(char *)(color)}
#define nxSTYLE(style)	    {nxtypeSTYLE|nxINT,nxstrSTYLE,(char *)(style)}
#define nxEND		    {0, NULL, NULL}

/* nxlist.c*/
/* dbl linked list data structure*/
typedef struct _nxlist {		/* LIST must be first decl in struct*/
	struct _nxlist *next;		/* next item*/
	struct _nxlist *prev;		/* previous item*/
} NXLIST, *PNXLIST;

/* dbl linked list head data structure*/
typedef struct _nxlisthead {
	struct _nxlist *head;		/* first item*/
	struct _nxlist *tail;		/* last item*/
} NXLISTHEAD, *PNXLISTHEAD;

/* field offset*/
#define NXITEM_OFFSET(type, field)    ((long)&(((type *)0)->field))

void * 	nxItemAlloc(unsigned int size);
void	nxListAdd(PNXLISTHEAD pHead,PNXLIST pItem);
void	nxListInsert(PNXLISTHEAD pHead,PNXLIST pItem);
void    nxListInsertAfter(PNXLISTHEAD pHead, PNXLIST pItem, PNXLIST pPrev);


void	nxListRemove(PNXLISTHEAD pHead,PNXLIST pItem);
#define nxItemNew(type)	((type *)nxItemAlloc(sizeof(type)))
#define nxItemFree(ptr)	free((void *)ptr)

/* return base item address from list ptr*/
#define nxItemAddr(p,type,list)	((type *)((long)p - NXITEM_OFFSET(type,list)))

/* utility routines*/
void	strzcpy(char *dst,char *src,int count);

/* iniread.c*/
int	IniGetString(char *section, char *key, char *defval, char *retbuf,
		int bufsiz, char *inifile);
void	IniEnumKeyValues(char *buf, void (*pfn)(char *,char *,int), int data);

/* Configuration file routines */


#define NX_CONFIG_MAIN  0
#define NX_CONFIG_ASSOC 1
#define NX_CONFIG_COUNT 2

int nxLoadConfigFile(char *filename, int index);
char* nxGetConfigFile(int index);
void nxFreeConfigFiles();
