/*
 * IPAQ.C
 * Copyright 2000, Century Software Embedded Technologies
 */

/* 
 * DESCRIPTION:  This file contains system specific routines
 *               for the Compaq Ipaq device.
 *               
 * AUTHOR:       First draft, Jordan Crouse, 11/27/00
 */

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <h3600_ts.h>

#include "nxsys.h"
#include "linux.h"
#include "nxcomm.h"
#include "linux_if.h"
#include "ipaq.h"

/******** NXSYS FUNCTIONS **********/

static unsigned char flite_brightness = 25;

static void ipaq_flite_control(enum flite_pwr power, 
			       unsigned char brightness)
{
   int             fd;
  FLITE_IN        bl;

  fd = open("/dev/h3600_ts", O_RDWR);

  if (fd == -1)
    return;

  /* Remember the brightness */
  flite_brightness = brightness;

  bl.mode = FLITE_MODE1;
  bl.pwr = power;
  bl.brightness = brightness;
  
  ioctl(fd, FLITE_ON, (void *) &bl);
  
  close(fd);
}

/*
 * ipaq_set_backlight(level, power)
 * set the Ipaq backlight to the new level, and set the power accordingly
 */

int ipaq_set_backlight(unsigned char bright, unsigned char power)
{
  enum flite_pwr pwr;

  if (power == NXSYS_BLITE_POWER_ON)
    pwr = FLITE_PWR_ON;
  else
    pwr = FLITE_PWR_OFF;

  /* If the brightness is zero, then turn off the backlite power anyway */

  if (bright == 0)
    pwr = FLITE_PWR_OFF;

  bright = (bright * 255) / 100;
  ipaq_flite_control(pwr, bright);
  return(NXSYS_OK);
}

int ipaq_get_battery(nxsys_battery_t *battery_struct)
{ 
  int             fd;
  BAT_DEV         bat;

  if (!battery_struct)
    return(NXSYS_ERROR);

  fd = open("/dev/h3600_ts", O_RDWR);

  if (fd == -1)
    return(NXSYS_ERROR);
  
  ioctl(fd, GET_BATTERY_STATUS, (void *) &bat);

  close(fd);
    
  battery_struct->flags = 0;
  
  if (bat.ac_status != H3600_AC_STATUS_AC_OFFLINE && bat.ac_status != H3600_AC_STATUS_AC_UNKNOWN)
    battery_struct->flags |= NXSYS_BATTERY_AC_ONLINE;
  
  if (bat.batt1_status != H3600_BATT_STATUS_NOBATT)
    battery_struct->flags |= NXSYS_BATTERY_ACTIVE;

  if (bat.batt1_status & H3600_BATT_STATUS_CHARGING)
    battery_struct->flags |= NXSYS_BATTERY_CHARGING;

  /* From the kernel */

  battery_struct->percent = (425 * bat.batt1_voltage) / 1000 - 298;
  battery_struct->battery_life = 300 * battery_struct->percent / 100;

  return(NXSYS_OK);
}

/******** NXCOMM FUNCTIONS **********/
 
int ipaq_set_nameserver(char *domain, char *search, unsigned long *addr, int acount)
{
  /* On the ipaq, we can only write to the /usr/local domain */
  /* so we have a symbolic link from /etc/resolv.conf to     */
  /* /usr/local/etc/resolv.conf, so thats the file we update here */

  return(linux_set_nameserver(RESOLV_FILE, domain, search, addr, acount));
}

int ipaq_get_net_value(char *key, char *ret)
{
	
	FILE *net_file;
	char buf[255];
	char *str_p;
	char ret_buf[255];
	int len = 0;
	int idx = 0;
	int nameserver = 0;
	int target = 0;

	if (0 == strcasecmp(key, "DOMAIN")) {
		goto space;
	}
	else if (0 == strcasecmp(key, "SEARCH")) {
		goto space;
	}
	else if (0 == strcasecmp(key, "DNS_1")) {
		goto space;
	}
	else if (0 == strcasecmp(key, "DNS_2")) {
		goto space;
	}
	else if (0 == strcasecmp(key, "DNS_3")) {
		goto space;
	}
	else {
		goto equal;
	}
	
	fclose(net_file);
	ret = "";

equal: //has an = for delim
	net_file = fopen(NET_FILE, "r");
	if (NULL == net_file) {
		fprintf(stderr, "Unbale to open file %s\n", NET_FILE);
		ret = "";
		return(NXSYS_ERROR);
	}
	while (NULL != fgets(buf, sizeof(buf) - 1, net_file)) {
		if ((str_p =  strstr(buf, key)) && ('#' != buf[0])) {
			ret_buf[0] = '\0';
			str_p = strstr(buf, "=");
			if (NULL != str_p) {
				str_p++;
				if ('\"' == *str_p) {
					str_p++;
					if ('\"' == *str_p || '\n' == *str_p) {
						fclose(net_file);
						ret = "";
						return(NXSYS_OK);
					}
				}
				len = strlen(str_p);
				for (idx = 0; idx < len; idx++) {
					if (('\"' != *str_p) && ('\n' != *str_p) && (NULL != str_p)) {
						ret_buf[idx] = *str_p;
						str_p++;
					}
					else {
						ret_buf[idx] = '\0';
						break;
					}
				}
			}	
			fclose(net_file);
			strncpy(ret, ret_buf, strlen(ret_buf) + 1);
			return(NXSYS_OK);
		}
	}

space: // has a " " space for a delim
	net_file = fopen(RESOLV_FILE, "r");
	if (NULL == net_file) {
		fprintf(stderr, "Unbale to open file %s\n", RESOLV_FILE);
		ret = "";
		return(NXSYS_ERROR);
	}
	while (NULL != fgets(buf, sizeof(buf) - 1, net_file)) {
		if (0 == strcmp(key, "DNS_1")) {
			target = 1;
			key = "nameserver";
		}
		if (0 == strcmp(key, "DNS_2")) {
			target = 2;
			key = "nameserver";
		}
		if (0 == strcmp(key, "DNS_3")) {
			target = 3;
			key = "nameserver";
		}
		if (0 == strncasecmp(key, buf, strlen(key))) {
			if (0 == strcmp(key, "nameserver")) {
				nameserver++;
				if (nameserver != target) {
			 		continue;
				}
			}	
			idx = 1;			
			str_p = strstr(buf, " ");
			if (NULL != str_p) {
				str_p++;
				len = strlen(str_p);
				for (idx = 0; idx < len; idx++) {
					ret_buf[idx] = *str_p;
					str_p++;
				}
			}
			ret_buf[idx-1] = '\0';
			fclose(net_file);
			strncpy(ret, ret_buf, strlen(ret_buf) + 1);
			return(NXSYS_OK);
		}
	}		
	return(NXSYS_OK);
}

int ipaq_write_dns_values(nxsys_dns_t dns_info)
{
	unsigned long dns_addr[2];
	int dns_count = 0;
	int ret;
	
	if (0 != strcmp("...", dns_info.str_dns_1)) {
 		if (0 != strcmp("", dns_info.str_dns_1)) {
			dns_addr[dns_count] = dns_info.long_dns_1;
			dns_count++;
		}
	}
	if (0 != strcmp("...", dns_info.str_dns_2)) {
		 if (0 != strcmp("", dns_info.str_dns_2)) {
			dns_addr[dns_count] = dns_info.long_dns_2;
			dns_count++;
		 }
	}
	if (0 != strcmp("...", dns_info.str_dns_3)) {
		 if (0 != strcmp("", dns_info.str_dns_3)) {
			dns_addr[dns_count] = dns_info.long_dns_3;
			dns_count++;
		 }
	}
	ret = ipaq_set_nameserver((char *)dns_info.domain, (char *)dns_info.search, dns_addr, dns_count);
	return (ret);
}

int ipaq_write_gateway_values(nxsys_ipaddr_str_t ip_info)
{
	FILE *net_file;

	net_file = fopen(IPAQ_GATEWAY_FILE, "w+");
	if (NULL == net_file) {
		fprintf(stderr, "Unbale to open file %s\n", IPAQ_GATEWAY_FILE);
   	 return(NXSYS_ERROR);
	}
	if (0 == strcmp("...", ip_info.gateway)) {
		fputs("" , net_file);
	}
	else {
		fputs(ip_info.gateway, net_file);
	}
	fclose(net_file);
   return(NXSYS_OK);
}

int ipaq_write_net_values(nxsys_ipaddr_str_t ip_info, nxsys_dns_t dns_info)
{
	FILE *net_file;
	char tmp_net[NXCOMM_IP_LEN];
	char *str_p = NULL;
	int count = 0;
	int num_per = 0;
	int idx = 0;

	net_file = fopen(NET_FILE, "w");
	if (NULL == net_file) {
		fprintf(stderr, "Unbale to open file %s\n", NET_FILE);
		return(NXSYS_ERROR);
	}
	fputs("# Network adapter configuration\n", net_file);
	fputs("#\n", net_file);
	fputs("# The address format is \"sceme,socket,instance,hwaddr\".\n", net_file);
	fputs("#\n", net_file);
	fputs("# Note: the \"network address\" here is NOT the same as the IP address.\"\n", net_file);
	fputs("# See the networking HOWTO.  In short, the network address is the IP\n", net_file);
	fputs("# address masked by the netmask.\n", net_file);
	fputs("#\n\n", net_file);
	fputs("#\n\necho \"network.opts: [$ADDRESS]\"\n\n", net_file);
	fputs("# defaults (need to set IP_ADDR at a minimum)\n\n", net_file);
	fputs("#Transceiver selection, for some cards -- see 'man ifport'\n", net_file);
	fputs("IF_PORT=\"\"\n", net_file);
	fputs("# Use DHCP? [y/n]\n", net_file);
	fputs("DHCP=\"",net_file);
	if (1 == ip_info.dhcp) {
		fputs("y\"\n", net_file);
		fputs("PUMP=\"y\"\n", net_file);
	}
	else {
		fputs("n\"\n", net_file);
		fputs("PUMP=\"n\"\n",net_file);
	}
	fputs("NET_MASK=\"", net_file);
	fputs(ip_info.netmask, net_file);
	fputs("\"\n", net_file);
	str_p = ip_info.netmask;
	while (NULL != str_p) {
		if (0 == strncmp("255", str_p, 3)) {
			count++;
		}
		else {
		 	break;
		}
		str_p = strstr(str_p, ".");
		if (NULL != str_p) {
			str_p++;
		}
	}		
	if (4 == count) {
		ip_info.subnet = ip_info.addr;
	}
	else if (0 == count) {
		ip_info.subnet = "";
	}
	else {
		sprintf(tmp_net, "%s", ip_info.addr);
   	for (idx = 0; idx <= strlen(tmp_net); idx++) {	
			if ('.' == tmp_net[idx]) {
				num_per++;
			}
			if (count == num_per) {
				ip_info.subnet = (char *)calloc(idx+1, sizeof(char));
				strncpy(ip_info.subnet, tmp_net, idx);
				break;
			}		  
		}
	}	

	num_per = 0;					  
	fputs("SUBNET=\"", net_file);
	fputs(ip_info.subnet, net_file);
	fputs("\"\n", net_file);
	fputs("NETWORK=\"$SUBNET", net_file);
	sprintf(tmp_net,"%s", ip_info.subnet);
	for (idx = 0; idx <= strlen(tmp_net); idx++) {
		if ('.' == tmp_net[idx]) {
			num_per++;
		}
	}
	if (0 == strlen(tmp_net) && 0 == num_per) {
		sprintf(tmp_net, "0.0.0.0");
	}
	else if (0 != strlen(tmp_net) && 0 == num_per) {
		sprintf(tmp_net, ".0.0.0");
	}
	else if (1 == num_per) {
		sprintf(tmp_net, ".0.0");
	}
	else if (2 == num_per) {
		sprintf(tmp_net, ".0");
	}
	else {
		sprintf(tmp_net,"");
	}
	fputs(tmp_net, net_file);
	fputs("\"\n", net_file);
	fputs("BROADCAST=\"", net_file);
	fputs(ip_info.broadcast, net_file);
	fputs("\"\n", net_file);
	fputs("# Gateway address for static routing\n", net_file);
	fputs("GATEWAY=\"", net_file);
	fputs(ip_info.gateway, net_file);
	fputs("\"\n", net_file);
	fputs("# Things to add to /etc/resolv.conf for this interface\n", net_file);
	fputs("DOMAIN=\"", net_file);
	fputs(dns_info.domain, net_file);
	fputs("\"\n", net_file);
	fputs("SEARCH=\"", net_file);
	fputs(dns_info.search, net_file);
	fputs("\"\n", net_file);
	fputs("DNS_1=\"", net_file);
	fputs(dns_info.str_dns_1, net_file);
	fputs("\"\n", net_file);
	fputs("DNS_2=\"", net_file);
	fputs(dns_info.str_dns_2, net_file);
	fputs("\"\n", net_file);
	fputs("DNS_3=\"", net_file);
	fputs(dns_info.str_dns_3, net_file);
	fputs("\"\n", net_file);
	fputs("# NFS mounts, should be listed in /etc/fstab\n", net_file);
	fputs("MOUNTS=\"\"\n", net_file);
	fputs("#For IPX interfaces, the frame type and network number\n", net_file);
	fputs("IPX_FRAME=\"\"\n", net_file);
	fputs("IPX_NETNUM=\"\"\n", net_file);
	fputs("# Extra stuff to do before shutting down the interface\n", net_file);
	fputs("start_fn () { return; }\n\n", net_file);
	fputs("case \"$ADDRESS\" in\n", net_file);
	fputs("*,*,*,*)\n", net_file);
	fputs("    ;;\n", net_file);
	fputs("esac", net_file);
	fclose(net_file);
	if (NULL != ip_info.subnet) {
#ifdef DEBUG
		fprintf(stderr, "in ipaq_write_net_values 07\n");
#endif
		free(ip_info.subnet);
	}
	
	ipaq_write_dns_values(dns_info);
	ipaq_write_gateway_values(ip_info);

	return(NXSYS_OK);
}
