/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: NISTTIME.CPP                                               *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <arpa/inet.h>
#include <errno.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include "nisttime.h"



/*
**
** Local Constant Definitions
**
*/
#define NIST_NTS_PORT 13



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/



/*
**
** Local Variable Declarations
**
*/
static char *server_list[] =
{
  "129.6.15.28, time-a.nist.gov, NIST, Gaithersburg, Maryland, USA",
  "129.6.15.29, time-b.nist.gov, NIST, Gaithersburg, Maryland, USA",
  "132.163.4.101, time-a.timefreq.bldrdoc.gov, NIST, Boulder, Colorado, USA",
  "132.163.4.102, time-b.timefreq.bldrdoc.gov, NIST, Boulder, Colorado, USA",
  "132.163.4.103, time-c.timefreq.bldrdoc.gov, NIST, Boulder, Colorado, USA",
  "128.138.140.44, utcnist.colorado.edu, University of Colorado, Boulder, Colorado, USA",
  "192.43.244.18, time.nist.gov, NCAR, Boulder, Colorado, USA",
  "131.107.1.10, time-nw.nist.gov, Microsoft, Redmond, Washington, USA",
  "209.0.72.7, nist1.datum.com, Datum, San Jose, California, USA",
  "216.200.93.8, nist1.dc.certifiedtime.com, Abovenet, Reston, Virginia, USA",
  "208.184.49.9, nist1.nyc.certifiedtime.com, Abovenet, New York, New York, USA",
  "208.185.146.41, nist1.sjc.certifiedtime.com, Abovenet, San Jose, California, USA",
  NULL
};
static char serverIP[16];
static int sock = -1;
static int UTCOffset = -7;



/*
**
** The following function will recover the time from the NIST and their
** atomic clock.
**
*/
static int getTime(nxsys_date_t *stamp)
{
  char buf[256],*tstr;
  int count,result;
  short value;
  struct in_addr address;
  struct sockaddr_in hostaddr;

  // recover the host IP address and port
  if(inet_aton(serverIP,&address) == 0)
    return(-1);

  // create the socket
  sock = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
  if(sock == -1)
    return(-2);

  // load the host's address
  hostaddr.sin_family = AF_INET;
  hostaddr.sin_port = htons(NIST_NTS_PORT);
  hostaddr.sin_addr = address;

  // connect the socket
  result = connect(sock,(struct sockaddr *)&hostaddr,sizeof(hostaddr));
  if(result == -1)
  {
    close(sock);
    sock = -1;
    return(-3);
  }

  // read the socket data
  result = recv(sock,buf,sizeof(buf),0);
  close(sock);
  sock = -1;

  // recover the year
  tstr = strstr(buf," ");
  ++tstr;
  value = ((*tstr & 0x0f) * 10) + (*(tstr + 1) & 0x0f);
  stamp->year = value + 2000;

  // recover the month
  value = ((*(tstr + 3) & 0x0f) * 10) + (*(tstr + 4) & 0x0f);
  stamp->month = value;

  // recover the date
  value = ((*(tstr + 6) & 0x0f) * 10) + (*(tstr + 7) & 0x0f);
  stamp->day = value;

  // recover the hour
  value = ((*(tstr + 9) & 0x0f) * 10) + (*(tstr + 10) & 0x0f);
  stamp->hour = value;

  // recover the minute
  value = ((*(tstr + 12) & 0x0f) * 10) + (*(tstr + 13) & 0x0f);
  stamp->minute = value;

  // recover the second
  value = ((*(tstr + 15) & 0x0f) * 10) + (*(tstr + 16) & 0x0f);
  stamp->second = value;

  // adjust for timezone offset
  if(UTCOffset < 0)
  {
    count = abs(UTCOffset);
    while(count--)
      nxsysDecrementHour(stamp);
  }
  else if(UTCOffset > 0)
  {
    count = UTCOffset;
    while(count--)
      nxsysIncrementHour(stamp);
  }

  // exit with no errors
  return(0);
}



/*
**
** This function will recover the current NIST time server name, IP
** address, and geographical location.
**
** The "result" parameter is a pointer to a buffer where the time
** server information will be stored.
**
** If successful, '0' is returned. If an error occurs during method
** execution, a non-zero value is returned that describes the error.
**
*/
int nxsysGetTimeServer(char *result)
{
  int count;

  // clear the result buffer
  *result = 0x00;

  // find the IP address in the server list
  count = 0;
  while(1)
  {
    if(server_list[count] == NULL)
      return(-1);
    if(strncmp(serverIP,server_list[count],strlen(serverIP)) == 0)
    {
      strcpy(result,server_list[count]);
      break;
    }
    ++count;
  }

  // exit with no errors
  return(0);
}



/*
**
** This function will set the IP address used for subsequent NIST time
** server queries.
**
** The "ipaddr" parameter is a pointer to a buffer that contains the
** text string that specifies the new IP address. The text string is
** in standard four-entry dotted notation.
**
** If successful, '0' is returned. If an error occurs during method
** execution, a non-zero value is returned that describes the error.
**
*/
int nxsysSetTimeServer(char *ipaddr)
{
  int count;

  // verify the IP address
  count = 0;
  while(1)
  {
    if(server_list[count] == NULL)
      return(-1);
    if(strncmp(ipaddr,server_list[count],strlen(ipaddr)) == 0)
    {
      strcpy(serverIP,ipaddr);
      break;
    }
    ++count;
  }

  // exit with no errors
  return(0);
}



/*
**
** This function will recover the current local time offset used to
** calculate local time.
**
** The number of hours offset from UTC (Universal Coordinated Time)
** is returned.
**
*/
int nxsysGetLocalOffset(void)
{
  return(UTCOffset);
}



/*
**
** This function will set the local time offset used to calculate
** local time.
**
** The "offset" parameter specifies the number of hours earlier or
** later than UCT for the local time. For example, set this to -7
** for Mountain Standard Time or -6 for Mountain Daylight Time.
**
** If successful, '0' is returned. If an error occurs during method
** execution, a non-zero value is returned that describes the error.
**
*/
int nxsysSetLocalOffset(int offset)
{
  // verify offset value
  if((offset < -12) || (offset > 12))
    return(-1);

  // save the new offset and exit with no errors
  UTCOffset = offset;
  return(0);
}



/*
**
** This function will recover the current date/time from the NIST, and
** initialize the specified timestamp with the date/time information.
**
** The "stamp" parameter is a pointer to the nxsys_date_t structure that
** will be loaded with the NIST date/time information.
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
*/
int nxsysGetDateTime(nxsys_date_t *stamp)
{
  return(getTime(stamp));
}



/*
**
** This function will recover the current date/time from the NIST, and
** format the result as an ASCIIZ string that is copied to the specified
** result buffer. The string format is specified as follows:
**
** format=0:   June 27, 2001
** format=1:   June 27, 2001, 10:58 AM
** format=2:   June 27, 2001, 10:58:30 AM
** format=3:   Wednesday, June 27, 2001
** format=4:   Wednesday, June 27, 2001, 10:58 AM
** format=5:   Wednesday, June 27, 2001, 10:58:30 AM
** format=6:   06/27/01
** format=7:   06/27/01 10:58a
** format=8:   06/27/01 10:58:30a
** format=9:   Wed 06/27/01
** format=10:  Wed 06/27/01 10:58a
** format=11:  Wed 06/27/01 10:58:30a
** format=12:  27 Jun 2001
** format=13:  27 Jun 2001 10:58 AM
** format=14:  27 Jun 2001 10:58:00 AM
** format=15:  Wed 27 Jun 2001
** format=16:  Wed 27 Jun 2001 10:58 AM
** format=17:  Wed 27 Jun 2001 10:58:30 AM
** format=18:  10:58 AM
** format=19:  10:58:30 AM
**
** The "format" parameter specifies the format of the date/time string
** and is chosen from the list above.
**
** If successful, '0' is returned. If an error occurs during method
** execution, a non-zero value is returned that describes the error.
**
*/
int nxsysGetDateTimeStr(int format, char *result)
{
  nxsys_date_t stamp;
  int res;

  // recover the current time from the NIST
  res = getTime(&stamp);
  if(res != 0)
    return(res);
  nxsysGetTimeDateString(&stamp,format,result);

  // exit with no errors
  return(0);
}



/*
**
** This function will recover the current date/time from the NIST, and
** convert it to the number of seconds that have elapsed since 12:00am
** on January 1, 1970 (UTC).
**
** The number of seconds elapsed is returned. If an error occurs during
** method execution, 0xFFFFFFFF (4294967295) is returned.
**
*/
unsigned int nxsysGetDateTimeValue(void)
{
  nxsys_date_t stamp;
  int result;
  struct tm tdata;

  // recover the current time from the NIST
  result = getTime(&stamp);
  if(result != 0)
    return(result);

  // initialize the tm structure
  tdata.tm_year = stamp.year - 1900;
  tdata.tm_mon = stamp.month - 1;
  tdata.tm_mday = stamp.day;
  tdata.tm_hour = stamp.hour;
  tdata.tm_min = stamp.minute;
  tdata.tm_sec = stamp.second;

  // recover the seconds since the epoch
  result = mktime(&tdata);

  // return result and exit with no errors
  return(result);
}
