/*******************************************************************************\
**
**	Header File:	cmdutil.h
**	Desc:			Header file for the generic wmcmd command utility functions
**	History:		05/08/01	J Webb
**								Initial Version
**
\*******************************************************************************/

#ifndef WMCMD_CMDUTIL_INCLUDED
#define	WMCMD_CMDUTIL_INCLUDED	1

#include <signal.h>
#include <sys/types.h>


#define		CMD_TIMEOUT			2									// For now a 5 sec timeout

#ifdef __cplusplus
extern "C" {
#endif

// Function prototypes

/*******************************************************************************\
**
**	Function:	int WCMDU_DidCmdFinish()
**	Desc:		Waits until command finishes, or CMD_TIMEOUT seconds before
**				returning
**	Accepts:	int error = The value of the error to report on failure
**				int timeout = The timeout value to wait
**	Returns:	int; 0 on success, or error on failure
**	
\*******************************************************************************/
extern int WCMDU_DidCmdFinish(int error, int timeout = 0);

/*******************************************************************************\
**
**	Function:	int WCMDU_ProcSimpleCmd()
**	Desc:		Processes the simple commands that don't require the passing of
**				any parameters between the shared mem segment
**	Accepts:	int command = the command (a WMCMD_XXXXX value -- see cmdblock.h)
**				int error = The error to report
**				int timeout = Number of seconds for timeout
**	Returns:	int; 0 on success, error - 1 on non-idle status, or error on timeout
**
\*******************************************************************************/
extern int WCMDU_ProcSimpleCmd(int command, int error, int timeout = 0);

/*******************************************************************************\
**
**	Function:	int WCMDU_StopWrApp()
**	Desc:		Issues a kill on the Running application's pid file
**	Accepts:	char *lockfile = Path of the pid lock file
**	Returns:	int; 0 on success, -1 on error (system call failure)
**
\*******************************************************************************/
extern int WCMDU_StopWrApp(char *lockfile, int signo = SIGINT);

/*******************************************************************************\
**
**	Function:	void WCMDU_UpdAppLckFile()
**	Desc:		Writes the pid of the newly executing wrapper app into the pid file
**	Accepts:	pid_t pid = Pid of the application
**				char *lockfile = Path of the pid lock file
**	Returns:	Nothing (void)
**
\*******************************************************************************/
extern void WCMDU_UpdAppLckFile(pid_t pid, char *lockfile);

/*******************************************************************************\
**
**	Function:	int WCMDU_WrAppRunning()
**	Desc:		Determines the wrapper application is currently running based on
**				the last known pid of it's execution
**	Accepts:	char * = Ptr to the path of the pid file
**	Returns:	int; 0 if process doesn't exist, 1 otherwise
**
\*******************************************************************************/
extern int WCMDU_WrAppRunning(char *lockfile);

#ifdef __cplusplus
}
#endif 

#endif // WMCMD_CMDUTIL_INCLUDED
