/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: plugin.h                                                   *
*                                                                      *
***********************************************************************/
#ifndef PLUGIN_H
#define PLUGIN_H



/*
**
** Imported "Include" Files
**
*/



/*
**
** Global Constant Definitions
**
*/
#define MAX_PLUGINS 16
#define MAX_FILENAME_LEN 256



/*
**
** Global Enumeration Definitions
**
*/



/*
**
** Global Structure Definitions
**
*/
typedef struct
{
  void *handle;
  char filename[MAX_FILENAME_LEN];
  int (*startupFunc)(void);
  int (*shutdownFunc)(void);
  int (*nameFunc)(char *);
  int (*versionFunc)(char *);
  int (*keywordFunc)(char *);
  int (*helpFunc)(void);
  int (*commandFunc)(int, char *[]);
}PluginEntry;



/*
**
** Global Variable Declarations
**
*/
extern PluginEntry plugin_list[MAX_PLUGINS];
extern int plugin_list_len;



/*
**
**  NAME: loadPlugins()
**
** USAGE: int loadPlugins(char *dir);
**
** DESCR: This function will load and register all plugins found in the
**        specified directory.
**
** PARMS: The "dir" parameter is a pointer to the buffer that contains the
**        directory to be searched for plugins.
**
** RETRN: If successful, '0' is returned. If an error occurs during function
**        execution, a non-zero value is returned that describes the error.
**
*/
int loadPlugins(char *);



/*
**
**  NAME: unloadPlugins()
**
** USAGE: int unloadPlugins(void);
**
** DESCR: This function will unload all plugins previously loaded by the
**        "pluginLoad()" function and free all associated memory.
**
** PARMS: None.
**
** RETRN: If successful, '0' is returned. If an error occurs during function
**        execution, a non-zero value is returned that describes the error.
**
*/
int unloadPlugins(void);

#endif /* PLUGIN_H */

