/********************************************************************************
  input/streamsrc_linux_http.hh

  purpose:
    A stream source that reads data from http source.

  notes:
    This code is Linux specific!

  to do:
   - This is alpha stage code!

  author(s):
   - Jeff Moellmer, Century Embedded Technologies, 2001

  modifications:
 ********************************************************************************
    Copyright (C) 1999  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
********************************************************************************/


#ifndef INPUT_STREAMSRC_LINUX_HTTP_HH
#define INPUT_STREAMSRC_LINUX_HTTP_HH

extern "C" {
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <netdb.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
}

#include "streamsrc.hh"
#include <iostream.h>

#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff
#endif

static const char *httpstr="http://";

class StreamSource_HTTP : public StreamSource
{
public:
  StreamSource_HTTP();
  ~StreamSource_HTTP();


  // HTTP specific methods

  void Init(const char* url=0);
  bool _readbuffer(char *buffer,int bytes);
  int  getbytedirect(void);
  bool eof(void);
  int  getblock(char *buffer,int size);

  int  getsize(void);
  int  getposition(void);
  void setposition(int pos);
  
  // overloaded methods

  int FillBuffer(BYTE* mem,DWORD maxlen);
  bool   MoreDataPending() const;
  
  bool   IsFiniteStream() const { return true; }
  uint64 AskStreamLength() const;
  
  bool   MaySeek() const { return false; } // TODO: of course seeking should be possible.
  uint64 AskCurrentPosition() const;
  uint64 SetCurrentPosition(uint64);

private:
  FILE *fp;
  int size;

  bool writestring(int fd, char* string);
  bool readstring(char *string, int maxlen, FILE *f);
  FILE *http_open(char *url);

  uint8  d_buf[2352];
  uint32 d_bufidx;
  uint32 d_buflen;
};

#endif
