/*******************************************************************************\
**
**	Module File:	cmdutil.cpp
**	Desc:			contains the generic command processing fucntions and "boiler
**					plate" code that up until now has been so repetive in the
**					plugins and wrapper apps...
**	History:		05/08/01	J Webb
**								Initial Version
**
\*******************************************************************************/

// Feature test switches


// System header files
#include <signal.h>
#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>


// Local header files
#include "cmdblock.h"
#include "cmdutil.h"
#include "sharedclient.h"


// Externally available functions

int
WCMDU_DidCmdFinish(int error, int timeout)
{
	int					rc = 0;										// Assume success
	time_t				tick;										// Time keeper variable

	// Validate timeout
	if (timeout == 0)
		timeout = CMD_TIMEOUT;

	// Sit and loop until either the command finishes, or a timeout occurs
	tick = time(NULL);
	while (1)
	{
		if (cmd_block->status == WMSTATUS_DONE)
			break;
		usleep(1000);
		if ( (tick + timeout) <= time(NULL))
		{
			rc = error;
			printf("Time Out Error\n");
			break;
		} // end of if
	} // end of while

	// Reset the command structure to known values
	cmd_block->command = WMCMD_NONE;
	cmd_block->status = WMSTATUS_IDLE;
	return (rc);
} // end of WCMDU_DidCmdFinish()

int
WCMDU_StopWrApp(char *lockfile, int signo)
{
	int					rc = 0;										// Assume success
	pid_t				wpid;										// Pid of wrapper app
	FILE				*plckfile;									// Lock file stream pointer

	// Open the lock file
	if ( (plckfile = fopen(lockfile, "r")) == NULL)
		return (rc);

	fscanf(plckfile, "%d", &wpid);
	fclose(plckfile);

	return (kill(wpid, signo));
} // end of WCMDU_StopWrApp()

int
WCMDU_WrAppRunning(char *lockfile)
{
	int					rc = 0;										// Assume that it is running
	pid_t				wpid;										// Pid of wrapper application
	FILE				*plckfile;									// Lock file stream pointer

	// Open the lock file
fprintf(stdout, "cmdutil:DEBUG opening %s to get pid.\n", lockfile);
	if ( (plckfile = fopen(lockfile, "rw")) == NULL)
		return (rc);

fprintf(stdout, "cmdutil:DEBUG Getting pid\n");
	fscanf(plckfile, "%d", &wpid);
	fclose(plckfile);

fprintf(stdout, "cmdutil:DEBUG Sending pid %d SIGNUL.\n", wpid);
	// Send this process a SIGNAL 0
	rc = kill(wpid, 0);

fprintf(stdout, "cmdutil:DEBUG rc=%d, return value=%d.\n", rc, !rc);
	return (!rc);
} // end of WCMDU_WrAppRunning()

	
int
WCMDU_ProcSimpleCmd(int command, int error, int timeout)
{
	if (timeout == 0)
		timeout = CMD_TIMEOUT;

	// Check to see if the command processor is ready to receive
	if (cmd_block->status != WMSTATUS_IDLE)
		return (error - 1);

	// Insert the command into the command processor
	cmd_block->command = command;
	cmd_block->status = WMSTATUS_START;

	return (WCMDU_DidCmdFinish(error, timeout));
} // end of WCMDU_ProcSimpleCmd()

void
WCMDU_UpdAppLckFile(pid_t pid, char *lockfile)
{
	FILE				*plckfile;									// Lock file

	if ( (plckfile = fopen(lockfile, "w")) != NULL)
	{
		fprintf(plckfile, "%d", pid);
		fclose(plckfile);
	} // end of if

	return;
} // end of WCMDU_UpdAppLckFile()
