/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: dispatcher.cpp                                             *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <fcntl.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include "errorbase.h"
#include <linux/fs.h>
#include <linux/videodev.h>
#include "plugin.h"
#include "status.h"
#include "state.h"



/*
**
** Local Constant Definitions
**
*/
#define MAX_PARM_LEN 64
#define VIDEO_DEVICE "/dev/video0"



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/



/*
**
** Local Variable Declarations
**
*/
static int call_level = 0;



/*
**
** This function will initialize the video system.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function must be called before any other function in the
** VAPI is called.
**
*/
static int videoOpen(void)
{
  char buf[256];
  int result,vhandle;
  struct v4l2_capability caps;
  struct v4l2_window window;

  vhandle = open(VIDEO_DEVICE,O_RDONLY);
  if(vhandle < 0)
  {
    VAPIStatusOutStr("No video device: /dev/video0");
    return(ERRORBASE_DISPATCHER + 0);
  }

  result = ioctl(vhandle,VIDIOC_QUERYCAP,&caps);
  if(result < 0)
  {
    VAPIStatusOutStr("VIDIOC_QUERYCAP failure");
    return(result);
  }

  close(vhandle);

  sprintf(buf,"Device: \"%s\":",caps.name);
  VAPIStatusOutStr(buf);
  sprintf(buf,
	  "Inputs:%d Outputs:%d Audios:%d",
	  caps.inputs,
	  caps.outputs,
	  caps.audios);
  VAPIStatusOutStr(buf);
  sprintf(buf,
	  "MaxWidth:%d MaxHeight:%d MinWidth:%d MinHeight:%d",
	  caps.maxwidth,
	  caps.maxheight,
	  caps.minwidth,
	  caps.minheight);
  VAPIStatusOutStr(buf);
  sprintf(buf,"MaxFrameRate:%d",caps.maxframerate);
  VAPIStatusOutStr(buf);

  // exit with no errors
  return(0);
}



/*
**
** This function will close down the video system and return all
** memory to the system.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
static int videoClose(void)
{
  // exit with no errors
  return(0);
}



/*
**
**
**
*/
static void help(char *command)
{
  char buf[256],keyword[256];
  int count;

  printf("\nUsage:\n");

  // see if keyword help was requested
  keyword[0] = 0x00;
  if(strcmp(command,"help") != 0)
  {
    strcpy(keyword,command + 5);
    count = 0;
    do
    {
      (plugin_list[count].keywordFunc)(buf);
      if(strcmp(buf,keyword) == 0)
      {
	printf("   ---------- %s commands\n",keyword);
	(plugin_list[count].helpFunc)();
	printf("\n");
	return;
      }
    }while(++count < plugin_list_len);
  }

  // display general help
  count = 0;
  do
  {
    (plugin_list[count].keywordFunc)(buf);
    printf("   ---------- %s commands\n",buf);
    (plugin_list[count].helpFunc)();
    printf("\n");
  }while(++count < plugin_list_len);
}



/*
**
**
**
*/
static void plugins(void)
{
  char name[256],version[256];
  int count;

  printf("\n");
  count = 0;
  do
  {
    (plugin_list[count].nameFunc)(name);
    (plugin_list[count].versionFunc)(version);
    printf("plugin #%d: \"%s\"\n",count + 1,name);
    printf("  version: %s\n",version);
    printf(" filename: %s\n\n",plugin_list[count].filename);
  }while(++count < plugin_list_len);
  printf("\n");
}



/*
**
** This function will parse the specified command string and
** call the associated module to execute the command.
**
** The "command" parameter is a pointer to a buffer that contains the
** command to be executed.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** 04/20/01 -- J Webb
**	Extended Functionality to allow the "current" plugin keyword to use
**  the current source module
*/
extern "C"
int dispatcher(char *command)
{

VAPIStatusOutStr("Dispatcher Entry.\n");
	VAPIStatusOutVal(call_level);
  char *argv[16],buf[256],*ptr;
  int argc,count,res,result;

printf("Coming into dispatcher with the command %s\n",command);
  // check for help command
  if(strncmp(command,"help",4) == 0)
  {
    help(command);
    return(0);
  }

  // check for plugins command
  if(strcmp(command,"plugins") == 0)
  {
    plugins();
    return(0);
  }

  // startup all modules
  if(call_level++ == 0)
    videoOpen();

  // recover first command line token
  argc = 1;
  argv[0] = (char *)malloc(MAX_PARM_LEN);
  strcpy(argv[0],strtok(command," ,"));

  // First command line token should be the name of the plugin to load, if
  // it is "current" it needs to copy the current value of state_source into
  // argv[0]
	if (!memcmp(argv[0], "current", 7))
	{
		memcpy(argv[0], state_source, MAX_PARM_LEN);
		{
			/* Debug message */
			char			buf[256];								/* Buffer for debug messages */

			sprintf(buf, "Switching \"current\" source to \"%s\"\n", argv[0]);
			VAPIStatusOutStr(buf);
		} /* end of temporary memory lifetime */
	} /* end of if */

  // recover additional command line tokens
  while(1)
  {
    ptr = strtok(NULL," ,");
    if(ptr == NULL)
      break;
    ++argc;
    argv[argc - 1] = (char *)malloc(MAX_PARM_LEN);
    strcpy(argv[argc - 1],ptr);
  }
  
  // dispatch command and parameters to proper module
  count = 0;
  do
  {
    (plugin_list[count].keywordFunc)(buf);
    if(strcmp(buf,argv[0]) == 0)
    {
      result = (plugin_list[count].commandFunc)(argc - 1,argv + 1);
      break;
    }
  }while(++count < plugin_list_len);
  if(count == plugin_list_len)
    result = ERRORBASE_DISPATCHER + 1;

  // do housekeeping
  count = 0;
  do
  {
    free(argv[count]);
  }while(++count < argc);

  // shutdown all modules
  if(--call_level == 0)
    videoClose();
VAPIStatusOutStr("Dispatcher Exit.\n");
  // exit with no errors
printf("Returning from the dispatcher\n");
  return(result);
}

