/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: idbugplugin.cpp                                            *
*                                                                      *
***********************************************************************/

/*
**
** Imported "Include" Files
**
*/
#include <ctype.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include "cmdblock.h"
#include "cmdutil.h"
#include "errorbase.h"
#include "overlay.h"
#include "srctuner.h"
#include "sharedclient.h"
#include "state.h"

#include <anlg_video.h>


/*
**
** Local Constant Definitions
**
*/
#define		TIMEOUT					5								// Time out value
#define		IMAGE_APP	"idbugvwr"
#define		IMAGE_APP_PATH	"/bin/idbugvwr"


/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char *plugin_name = "IdBug image handler";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "idbug";
static int	g_RestartFlg = -1;

/* ANSI Function prototypes */
static int RestartIdBug(int chanId);
static int SetIdBug(int chanId);
static int StopIdBug(int chanId);

/*******************************************************************************\
**
**	Function:	static int RestartIdBug()
**	Desc:		Kills the current invocation and restarts, on change of some
**				parameter (either position or transparencey level)
**	Accepts:	int chanId = Channel id
**	Returns:	int 0 on success, non-zero otherwise
**
\*******************************************************************************/
static int
RestartIdBug(int chanId)
{
	int					result = 0;									// Result

	if (state_idbug[chanId].pid > 0 && strlen(state_idbug[chanId].idFile))
	{
		if ( (result = StopIdBug(chanId)) == 0)
		{
			state_idbug[chanId].pid = 0;
			result = SetIdBug(chanId);
		} // end of if
	} // end of if 

	return (result);
} // end of RestartIdBug()

/*******************************************************************************\
**
**	Function:	static int SetIdBug()
**	Desc:		Handles the nsc middleware api calls to enable to the idbug
**	Accepts:	int chanId = the alpha channel to control (0-2)
**	Returns:	int 0 (VMW_OK) on success or VMW_XXX error value otherwise
**
\*******************************************************************************/
static int
SetIdBug(int chanId)
{
	char				*idargv[7];									// Argument vector passed to exec()
	int					i,											// Loop iterator
						maxvectsz;									// Maximum vector size
	pid_t				cpid;										// Childs pid

	if  (state_idbug[chanId].pid > 0)
		return (ERRORBASE_IDBUG + 1);
	
	// Allocate memory for the argument vector
	maxvectsz = strlen(state_idbug[chanId].idFile) + 1;
	for (i = 0; i < 7 - 1; i++)
	{
		if ( (idargv[i] = (char *)calloc(maxvectsz, sizeof(char))) == NULL)
		{
			for (int j = 0; j < i; j++)
				free (idargv[j]);
			return (-1);
		} // end of if 
	} // end of for

	// Fill in the argument vector
	strcpy(idargv[0], IMAGE_APP);
	sprintf(idargv[1], "%d", state_idbug[chanId].xPos);
	sprintf(idargv[2], "%d", state_idbug[chanId].yPos);
	sprintf(idargv[3], "%d", chanId);
	sprintf(idargv[4], "%d", state_idbug[chanId].transLvl);
	strcpy(idargv[5], state_idbug[chanId].idFile);
	idargv[6] = NULL;

	// Exec
	if ( (cpid = fork()) == 0)
	{
		// child process
		execv(IMAGE_APP_PATH, idargv);
		return (-1);
	} // end of if

	for (i = 0; i < 7 - 1; free(idargv[i++]));

	// Update the state_information
	state_idbug[chanId].pid = cpid;

	return (0);
} // end of SetIdBug()

/*******************************************************************************\
**
**	Function:	static int StopIdBug()
**	Desc:		Sends the process associated with the current invocation of the
**				idbug a SIGUSR1 signal (which will terminate nicely)
**	Accepts:	int chanId = The channel to close remove image from
**	Returns:	int; 0 on success -1 on error
**
\*******************************************************************************/
static int
StopIdBug(int chanId)
{
	int					rc = 0;

	/*
	**	TODO:	See if there is a way to force state_idbug[chanId].pid to become
	**	a child of this process, and execute a waitpid to guarantee that the
	**			child dies
	*/

	if (state_idbug[chanId].pid > 0)
	{
		// Send the kill signal (any cleaning up of pid/filename is done the
		// callers responsibility.
		rc = kill(state_idbug[chanId].pid, SIGUSR1);
	} // end of if

	return (rc);
} // end of StopIdBug


/*
**
** Command: position channel x y
**
*/
static int positionFunc(int argc, char *argv[])
{
	int result = 0;
	int					chanId;										// The channel id

	// Get the channel id
	if (argc == 3)
	{
		if ( (chanId = atoi(argv[0])) < 0 || chanId > 2)
			chanId = 0;

		state_idbug[chanId].xPos = atoi(argv[1]);
		state_idbug[chanId].yPos = atoi(argv[2]);
		g_RestartFlg = chanId;
	} // end of if

  return(result);
}

/*
**
** Command: setlevel channel level (0-255)
**
*/
static int levelFunc(int argc, char *argv[])
{
	int					result = 0,									// Result
						chanId;										// The alpha channel id

	// Get the channel id
	if (argc == 2)
	{
		if ( (chanId = atoi(argv[0])) < 0 || chanId > 2)
			chanId = 0;


		state_idbug[chanId].transLvl = atoi(argv[1]);
		state_idbug[chanId].transLvl &= 0xff;
		g_RestartFlg = chanId;
	} // end of if

	return (result);
} // end of levelFunc


/*
**
**	Command:	add channel filename
**
*/
static int
addFunc(int argc, char *argv[])
{
	int					result = 0,									// Result
						chanId;										// Alpha Channel Id

	// Get the channel id
	if (argc == 2)
	{
		if ( (chanId = atoi(argv[0])) < 0 || chanId > 2)
			chanId = 0;
			

		// Get the filename
		if (strcmp(argv[1], state_idbug[chanId].idFile))
		{
			if (state_idbug[chanId].pid > 0)
			{
				// Stop the previous invocation of this
				StopIdBug(chanId);
				state_idbug[chanId].pid = 0;
			} // end of if
			strcpy(state_idbug[chanId].idFile, argv[1]);
		} // end of if

		if (strlen(state_idbug[chanId].idFile))
			if (SetIdBug(chanId) != 0)
				result = ERRORBASE_IDBUG + 2;
	} // end of if

	return (result);
} // end of addFunc()


/*
**
**	Command:	remove channel
**
*/
static int
removeFunc(int argc, char *argv[])
{
	int				result = 0,										// Result value
					chanId;											// Alpha Channel Id

	if (argc >= 1)
	{
		if ( (chanId = atoi(argv[0])) < 0 || chanId > 2)
			chanId = 0;

		if (state_idbug[chanId].pid)
			if (StopIdBug(chanId) != 0)
				result = ERRORBASE_IDBUG + 3;
		state_idbug[chanId].pid = 0;
		memset(state_idbug[chanId].idFile, 0, sizeof(state_idbug[chanId].idFile));
	} // end of if
	
	return (result);
} // end of removeFunc()

/*
**
** Command: info
**
*/
static int infoFunc(int argc, char *argv[])
{
	for (int i = 0; i < 3; i++)
	{
		printf("idbug[%d]: xpos:%d ypos:%d lvl:%d pid:%d file:%s\n", i,
					state_idbug[i].xPos, state_idbug[i].yPos, state_idbug[i].transLvl,
					state_idbug[i].pid, state_idbug[i].idFile);
	} // end of for
return(0);
}



/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "position",positionFunc,
  "setlevel", levelFunc,
  "add", addFunc,
  "remove", removeFunc,
  "info",infoFunc,
  NULL,NULL
};



/*
**
** This function will initialize the idbug module and prepare it
** for subsequent operations.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function will closeout the idbug module and shut it down.
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function will return the subsystem identification string
** for the idbug module ("idbug").
**
** The "result" parameter is a pointer to the buufer where the
** subsystem identification string will be stored.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
** This function will display the commands available for the idbug
** module.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s position chan x y (-1/-2 align x with left/right safetitle)\n",plugin_keyword);
  printf("                        (-3/-4 align y with top/bottom safetitle)\n");
  printf("   %s setlevel chan value\n",plugin_keyword);
  printf("   %s add chan image_file\n",plugin_keyword);
  printf("   %s remove chan \n",plugin_keyword);
  printf("   %s info\n",plugin_keyword);
  return(0);
}



/*
**
** This function will execute the specified idbug module command.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result = 0;

  // Set the g_RestartFlg to 0
	g_RestartFlg = -1;

  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_TUNER + 5;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

	if (g_RestartFlg > -1 && result == 0)
		RestartIdBug(g_RestartFlg);
  
  // return result and exit
fprintf(stdout, "Result of idbug command=%d\n", result);
  return(result);
}
