/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: mpegplugin.cpp                                             *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "audio.h"
#include "cmdblock.h"
#include "dispatcher.h"
#include "errorbase.h"
#include "sharedserver.h"
#include "srcmpeg.h"
#include "state.h"

#include <anlg_video.h>


/*
**
** Local Constant Definitions
**
*/
#define TIMEOUT 1



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char *plugin_name = "Hardware MPEG Support";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "mpeg";



/*
**
**
**
*/
static int loadFrame(void)
{
  int tick;

  // make sure cmd_block is idle
  if(cmd_block->status != WMSTATUS_IDLE)
    return(ERRORBASE_MPEG + 10);

  // issue the stop command
  cmd_block->command = WMCMD_VIEWER_FRAME_LOAD;
  strcpy(cmd_block->parm1,"Dialog");
  strcpy(cmd_block->parm2,"dialogstream.html");
  cmd_block->status = WMSTATUS_START;

  // wait for the command to finish
  tick = time(NULL);
  while(1)
  {
    if(cmd_block->status == WMSTATUS_DONE)
      break;
    usleep(1000);
    if((tick + TIMEOUT) <= time(NULL))
    {
      cmd_block->command = WMCMD_NONE;
      cmd_block->status = WMSTATUS_IDLE;
      //      return(ERRORBASE_MPEG + 11);
      return(0);
    }
  }
  cmd_block->command = WMCMD_NONE;
  cmd_block->status = WMSTATUS_IDLE;
  
  return(0);
}



/*
**
** Command: source
**
*/
static int sourceFunc(int argc, char *argv[])
{
	int					mpgrc;

  if(VAPISrcMPEGSelect() != 0)
    return(ERRORBASE_MPEG + 0);

	VMW_AVS_VideoInit();
	VMW_OVL_EnableOverlay(0x00f000f0, 0x00f0f0f0, 0);
	mpgrc = VIDEO_INPUT_MPEG_DECODER;
	VMW_AVS_SetCurrentVideoInput(&mpgrc);
	VMW_AVS_VideoClose();
	SetAudio(AV_CHAN_MPEG,0,0);
  strcpy(state_source,plugin_keyword);
  loadFrame();
  return(0);
}



/*
**
** Command: stop
**
*/
static int stopFunc(int argc, char *argv[])
{
  int tick;

  // make sure cmd_block is idle
  if(cmd_block->status != WMSTATUS_IDLE)
    return(ERRORBASE_MPEG + 1);

  // issue the stop command
  cmd_block->command = WMCMD_MPEG_STOP;
  cmd_block->status = WMSTATUS_START;

  // wait for the command to finish
  tick = time(NULL);
  while(1)
  {
    if(cmd_block->status == WMSTATUS_DONE)
      break;
    usleep(1000);
    if((tick + TIMEOUT) <= time(NULL))
    {
      cmd_block->command = WMCMD_NONE;
      cmd_block->status = WMSTATUS_IDLE;
      return(ERRORBASE_MPEG + 2);
    }
  }
  cmd_block->command = WMCMD_NONE;
  cmd_block->status = WMSTATUS_IDLE;
  
  return(0);
}



/*
**
** Command: play
**
*/
static int playFunc(int argc, char *argv[])
{
  int tick;

  // make sure cmd_block is idle
  if(cmd_block->status != WMSTATUS_IDLE)
    return(ERRORBASE_MPEG + 3);

  // issue the stop command
  strcpy(cmd_block->parm1,argv[0]);
  
  // start the mpegplayer process
  if ( fork() == 0 ) /* child */
    {
        execl( "/bin/mpegplayer", "mpegplayer", "", NULL);
    }

  cmd_block->command = WMCMD_MPEG_PLAY;
  cmd_block->status = WMSTATUS_START;

  // wait for the command to finish
  tick = time(NULL);
  while(1)
  {
    if(cmd_block->status == WMSTATUS_DONE)
      break;
    usleep(1000);
    if((tick + TIMEOUT) <= time(NULL))
    {
      cmd_block->command = WMCMD_NONE;
      cmd_block->status = WMSTATUS_IDLE;
      return(ERRORBASE_MPEG + 4);
    }
  }
  cmd_block->command = WMCMD_NONE;
  cmd_block->status = WMSTATUS_IDLE;
  
  return(0);
}



/*
**
** Command: pause
**
*/
static int pauseFunc(int argc, char *argv[])
{
  int tick;

  // make sure cmd_block is idle
  if(cmd_block->status != WMSTATUS_IDLE)
    return(ERRORBASE_MPEG + 5);

  // issue the stop command
  cmd_block->command = WMCMD_MPEG_PAUSE;
  cmd_block->status = WMSTATUS_START;

  // wait for the command to finish
  tick = time(NULL);
  while(1)
  {
    if(cmd_block->status == WMSTATUS_DONE)
      break;
    usleep(1000);
    if((tick + TIMEOUT) <= time(NULL))
    {
      cmd_block->command = WMCMD_NONE;
      cmd_block->status = WMSTATUS_IDLE;
      return(ERRORBASE_MPEG + 6);
    }
  }
  cmd_block->command = WMCMD_NONE;
  cmd_block->status = WMSTATUS_IDLE;
  
  return(0);
}

/*******************************************************************************\
**
**	Function:	int volumeupFunc()
**	Desc:		Increments the volume level by 1 unit
**	Acecpts:	int argc = number of args
**				char *argv[] = Arguments
**	Returns:	int; 0 on success, or an ERRORBASE error code
**
\*******************************************************************************/
static int
volumeupFunc(int argc, char *argv[])
{
	SetAudio(AV_CHAN_MPEG, 0, 1);
	return (0);
} /* end of volumeupFunc() */

/*******************************************************************************\
**
**	Function:	int volumedownFunc()
**	Desc:		Decrements the volume level by 1 unit
**	Accepts:	int argc = number of args
**				char *argv[] = Arguments
**	Returns:	int; 0 on success, or an ERRORBASE eror code 
**
\*******************************************************************************/
static int
volumedownFunc(int argc, char *argv[])
{
	SetAudio(AV_CHAN_MPEG, 0, -1);
	return (0);
} /* end if volumedownFunc() */

/*******************************************************************************\
**
**	Function:	int volumemuteFunc()
**	Desc:		Mutes the volume
**	Accepts:	int argc = Number of args
**				char *argv[] = Arguments
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumemuteFunc(int argc, char *argv[])
{
	fprintf(stderr, "Muting mpeg\n");
	SetAudio(AV_CHAN_MPEG, 1, 0);
	return (0);
} /* end of volumemuteFunc() */

/*******************************************************************************\
**
**	Funciton:	int volumeunmuteFunc()
**	Desc:		Unmutes the volume
**	Accepts:	int argc = Number of args
**				char *argv[] = Arguments
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumeunmuteFunc(int argc, char *argv[])
{
	fprintf(stderr, "Unmuting mpeg\n");
	SetAudio(AV_CHAN_MPEG, 0, 0);
	return (0);
} /* end of volumeunmuteFunc() */

/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "source",sourceFunc,
  "stop",stopFunc,
  "play",playFunc,
  "pause",pauseFunc,
  "volup", volumeupFunc,
  "voldn", volumedownFunc,
  "mute", volumemuteFunc,
  "unmute", volumeunmuteFunc,
  NULL,NULL
};



/*
**
** This function is called once, just after the plugin has been loaded
** and all plugin entrypoints have been registered. This function will
** be called before any other plugin entrypoint functions are called.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function is called once, just before the plugin is unloaded.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
** This function is called to recover the plugin's name/description.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** name/description will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
** This function is called to recover the plugin's internal version number.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** internal version number will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function is called to recover the plugin's command keyword.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** command keyword will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
** This function is called to display this plugin's command set.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s source\n",plugin_keyword);
  printf("   %s stop\n",plugin_keyword);
  printf("   %s play file\n",plugin_keyword);
  printf("   %s pause\n",plugin_keyword);
  printf("   %s mute\n",plugin_keyword);
  printf("   %s unmute\n",plugin_keyword);
  printf("   %s volup\n",plugin_keyword);
  printf("   %s voldn\n",plugin_keyword);
  return(0);
}



/*
**
** This function is called to execute one of the plugin's commands.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result;

  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_MPEG + 7;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

  // return result and exit
  return(result);
}
