/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Netstream Plugin Files
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
*/


/******************************************************************************
**									     **	
**			Program Name: netstrplayer.cpp 		   	     **
**		   Authors:  Lakshmi Gopinath and Chitra Shankaran	     **
**			   National Semiconductors        		     ** 
**				  Bangalore				     **
**									     **
******************************************************************************/

/*
**
** System Include Headers
**
*/
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

// National middleware includes
#include <nscstrapi.h>

/* Plugin includes */
#include "state.h"
#include "status.h"
#include "sharedclient.h"
#include "netstrerror.h"

/*
**
** Local Constant Definitions
**
*/
#define TRUE 1
#define FALSE 0

#define		NET_ACTION_PLAY					0x0001		
		//NetStr in play Mode	
#define 	NET_ACTION_PAUSE				0x0002
		//NetStr in Pause Mode
#define 	NET_ACTION_LOOP					0x0004
		//NetStr normal play mode
#define		NET_ACTION_MUTE					0x0008 
		//NetStr in Mute
#define 	NET_ACTION_FORWARD				0x0010
		//NetStr in Fast Forward Mode
#define 	NET_ACTION_REVERSE				0x0020
		//NetStr in Reverse Mode
#define 	NET_ACTION_LEFTAUDIO				0x0040
		//NetStr in Set left Audio Mode
#define 	NET_ACTION_RIGHTAUDIO				0x0080
		//NetStr in Set Right Audio Mode
#define 	NET_EXIT					0x0100
		//Exit from NetStr
#define 	NET_ACTION_STOP					0x0200
		//Netstr in stop mode
#define 	NET_ACTION_INIT					0x8000
		//Netstr in Init mode 

typedef unsigned short netflag_t;

/*
**
** Static Functions Prototype
**
*/
static int  ProcessInit(netflag_t *);
static int  ProcessPlay(netflag_t *);
static int  ProcessStop(netflag_t *);
static int  ProcessFastForward(netflag_t *);
static int  ProcessReverse(netflag_t *);
static int  ProcessPause(netflag_t *);
static int  ProcessLoop(netflag_t *);
static int  ProcessMute(netflag_t *);
static int  ProcessSetLeftAudio(netflag_t *,int);
static int  ProcessSetRightAudio(netflag_t *,int);
static int  ProcessSetRightAudio(netflag_t *,int);
static int  ProcessDisconnect(netflag_t *,int);

/**********************************************************************************
**		Syntax:		waitForCommand(void)			         **
**									         **
**	 	Parameters:	None						 **
**								  		 **
**		Returns:	The value of the command returned		 **
**										 **
**		Comments:	None						 **
**										 **
**********************************************************************************/
static int waitForCommand(void)
{
 	int cmd;

	 while(1)
  	 {
		usleep(1000);
		if(cmd_block->status != WMSTATUS_START)
      			continue;

		// Set the value, in case something else comes along and over-rides this
		// Note:  The above shouldn't happen, but just in case...
		cmd = cmd_block->command;
		if (	(cmd == WMCMD_STR_PLAY ) ||
			(cmd == WMCMD_STR_CONNECT) ||
			(cmd == WMCMD_STR_DISCONNECT) ||
			(cmd == WMCMD_STR_QUIT) ||
			(cmd == WMCMD_STR_STOP) ||
			(cmd == WMCMD_STR_PAUSE) ||
			(cmd == WMCMD_STR_MUTE) ||
			(cmd == WMCMD_STR_LOOP) ||
			(cmd == WMCMD_STR_FWD) ||
			(cmd == WMCMD_STR_REV) ||
			(cmd == WMCMD_STR_LEFTAUDIO) ||
			(cmd == WMCMD_STR_RIGHTAUDIO)	) 
		{
			break;
		} // end of if 
  	}
	cmd_block->status = WMSTATUS_DONE;
  	while(cmd_block->status != WMSTATUS_IDLE);
	return(cmd);
} /* end of wait for command */


/**********************************************************************************
**		Syntax:		ProcessInit(netflag_t *stateflag)	         **
**									         **
**	 	Parameters:	pointer to stateflag of type short               **
**								  		 **
**		Returns:	1 for success					 **
**				error code in case of error			 **	
**										 **
**		Comments:	server IP address is hardcoded 			 **
**										 **
**********************************************************************************/
static int 
ProcessInit(netflag_t *stateflag)
{
	char* files,*temp;
	char ip[50];
	char file_path[80];	
	char dir_path[256];
	FILE* fd;
	int i,rc = -1;
	
	*stateflag |= NET_ACTION_STOP; // as the stream is not playing

	// The file 'bin/netsrin' contains the IP address of the server, the    
	// path of the directory that the client wants to select on the server  
	// and the path of the file within that directory followed by a new     
	// line character						        

	if((fd = fopen("/bin/netstrin","r")) == NULL)
	{
		printf("Unable to open /bin/netstrin \n");
		return NETSTR_FLAG_MATCH_ERR;
	}
	
	bzero(ip,sizeof(ip));
	bzero(dir_path,sizeof(dir_path));
	bzero(file_path,sizeof(file_path));

	fscanf(fd,"%s",ip);
	ip[strlen(ip)] = '\0';
	fscanf(fd,"%s",dir_path);
	dir_path[strlen(dir_path)] = '\0';
	fscanf(fd,"%s",file_path);
	file_path[strlen(file_path)] = '\0';
	fclose(fd);

	printf("IP number read %s \n",ip);
	printf("directory  chosen: %s \n",dir_path);
	printf("file path chosen: %s \n",file_path);

	// initialise only when not initialised
	// connect to server, set the server directory, get the file list
	// of the directory, select the fileto be played and open the file
	// that has been selected.
	if(!((*stateflag) & NET_ACTION_INIT)) 	
	{
		if(NscStrHook2Srv(ip) == 0)
			return NETSTR_HOOK2SRV_ERR;
		(*stateflag) |= NET_ACTION_INIT; //turn init bit on
			//above suggests that client is connected to server
		if(NscStrSetDirectory(dir_path))
			printf("Setting Directory... \n");
		sleep(1);	
		rc = 0;
        	rc = NscStrGetNoFiles();
		sleep(1);
		// Check to see if the Directory you set is a valid One
		// The middleware API returns 1 in case it is valid 
		// To be done only for the linux mpeg server
		if(NscStrCheckSelectFile() == 0)
			return NETSTR_DIR_ERR;	
		
		//else check for the File List obtained from the server
		if(rc == -1)
			return NETSTR_GET_NOFILES_ERR;
		fprintf(stdout,"Number of files: %d\n",rc);
		files = (char*) malloc(80*rc);
		if(!files)
			return NETSTR_MALLOC_ERR;
		temp = (char*) malloc(80 * sizeof(char));
		if(!temp)
			return NETSTR_MALLOC_ERR;

		if(NscStrGetFileList(files,80*rc) == 0)
			return NETSTR_GET_FILELIST_ERR;
		fprintf(stdout,"File List Received\n");
		for(i=0; i<rc; i++)
		{
			sscanf(files+(80*i),"%s",temp);
			fprintf(stdout,"%s\n",temp);
		}	

		if(NscStrSelectFile(file_path) == 0)
			return NETSTR_SELECT_FILE_ERR;
		if(NscStrOpenCurrentFile()== 0)
			return NETSTR_OPEN_CURRENT_FILE_ERR;
		sleep(1);

		// Check to see if the file you selected is a valid one
		// Middleware API returns 1 in case it is valid
		// To be done only for the linux mpeg server
		if(NscStrCheckSelectFile() == 0)
			return NETSTR_FILE_ERR;	

		//free allocated memory before returning
		free(temp);
		free(files);
		return 1;
	} //end of if
	return NETSTR_FLAG_MATCH_ERR;
	
} /* end of ProcessInit */

/**********************************************************************************
**              Syntax:         ProcessDisconnect(netflag_t *stateflag)          **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**		Returns:	1 for success					 **
**				error code in case of error			 **	
**                                                                               **
**              Comments:       None 						 **
**                                                                               **
**********************************************************************************/
static int ProcessDisconnect(netflag_t * stateflag)
{
	int rc=0;
	printf("Disconnecting the stream\n");
	//Disconnect only if connected
	if( (*stateflag & NET_ACTION_INIT))
	{
		//Stop the stream if has not yet been stopped
		if(!((*stateflag) & NET_ACTION_STOP))
		{	// if the stream is playing
			if((rc = ProcessStop(stateflag)) != 1)
				return rc; 
		}
		sleep(1);
		//Disconnect from the server
	        if(NscStrUnHook2Srv())
		{
			sleep(1);
			(*stateflag) = 0; //reset all bits
			printf("Unhooked the server \n");
	        	return 1;
		}
		else
			return NETSTR_DISCONNECT_ERR;
	}
	return NETSTR_FLAG_MATCH_ERR;
} /* end of ProcessDisconnect */

/**********************************************************************************
**              Syntax:         ProcessQuit(netflag_t *stateflag)                **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**		Returns:	1 						 **
**										 **	
**              Comments:       None 						 **
**                                                                               **
**********************************************************************************/
static int ProcessQuit(netflag_t * stateflag)
{
	int rc;
	//Disconnect from server if it has not yet been
	if((*stateflag) & NET_ACTION_INIT)
	{
		rc = ProcessDisconnect(stateflag);
		if(rc != 1)
			printError(rc);
	}
	//Quit process whether Disconnect fails or succeeds
	if(NscStrQuit())
	sleep(1);
	(*stateflag) |= NET_EXIT;
        return 1;
} /* end of ProcessQuit

/**********************************************************************************
**              Syntax:         ProcessPlay(netflag_t *stateflag)                **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**		Returns:	1 for success					 **
**				error code in case of error			 **	
**                                                                               **
**              Comments:       None 						 **
**                                                                               **
**********************************************************************************/
static int
ProcessPlay(netflag_t *stateflag)
{
        int i;
	//proceed only if the client's connected to the server
	if((*stateflag) & NET_ACTION_INIT)
	{
		if(!((*stateflag) & NET_ACTION_PLAY) && 
		   !((*stateflag) & NET_ACTION_PAUSE))
		{
			if(NscStrPlay())
			{
				*stateflag |= NET_ACTION_PLAY;//set play bit on
				//Ensure that pause, fwd and rew bits are off
				*stateflag &= ~(NET_ACTION_PAUSE);
				*stateflag &= ~(NET_ACTION_REVERSE);
				*stateflag &= ~(NET_ACTION_FORWARD);
				*stateflag &= ~(NET_ACTION_STOP);
			}
			else
				return NETSTR_PLAY_ERR;
		}
		else if((*stateflag) & NET_ACTION_PAUSE)
		{	
			if(NscStrPause())
			{
				*stateflag &= ~(NET_ACTION_PAUSE);
				*stateflag &= ~(NET_ACTION_STOP);
				*stateflag |= NET_ACTION_PLAY;//turn play bit on
			}
			else
				return NETSTR_RESUME_PLAY_ERR;
		}
		else
			return NETSTR_FLAG_MATCH_ERR;
		return 1;
	}
	//if netstr in not initialised
	return NETSTR_NO_INIT_ERR;
} /* end of ProcessPlay */

/**********************************************************************************
**              Syntax:         ProcessStop(netflag_t *stateflag)                **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**              Returns:        1 for success                                    **
**                              error code in case of error                      **
**                                                                               **
**              Comments:       None						 ** 
**                                                                               **
**********************************************************************************/
static int
ProcessStop(netflag_t *stateflag)
{
	//If stream in mute, unmute before stopping
	if((*stateflag) & NET_ACTION_MUTE)
		ProcessMute(stateflag);

	//If stream in pause mode, play it before stopping because hardware
	//does not stop stopping of the stream in pause mode
	if((*stateflag) & NET_ACTION_PAUSE)
	{
		if(!NscStrPause())
			return NETSTR_RESUME_PLAY_ERR;
		else
		{
			(*stateflag) &= ~(NET_ACTION_PAUSE) ;
			(*stateflag) &= ~(NET_ACTION_STOP) ;
			(*stateflag) |= NET_ACTION_PLAY;
			sleep(1);
		}
	}
	//Stop the stream for the following conditions	
	if( ((*stateflag) & NET_ACTION_PLAY) ||
		 ((*stateflag) & NET_ACTION_FORWARD) ||
		 ((*stateflag) & NET_ACTION_REVERSE) ||
		 ((*stateflag) & NET_ACTION_LOOP) )

	{
		if(!NscStrStop())
			return NETSTR_STOP_ERR;
		else
		{
			printf("Stopping the Stream\n");
			//set the stop bit alone and reset all other bits
			(*stateflag) |= NET_ACTION_STOP;
			(*stateflag) &= ~(NET_ACTION_PLAY);
			(*stateflag) &= ~(NET_ACTION_FORWARD);
			(*stateflag) &= ~(NET_ACTION_REVERSE);
			(*stateflag) &= ~(NET_ACTION_PAUSE);
			(*stateflag) &= ~(NET_ACTION_MUTE);
			return 1;
		}
	}
	else
		return NETSTR_FLAG_MATCH_ERR;
} /* end of ProcessStop */

/**********************************************************************************
**              Syntax:         ProcessPause(netflag_t *stateflag)               **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**              Returns:        1 for success                                    **
**                              error code in case of error                      **
**                                                                               **
**              Comments:       None                                             **
**                                                                               **
**********************************************************************************/
static int
ProcessPause(netflag_t *stateflag)
{
	//Proceed only if in play mode
	//Pause if playing
	if((*stateflag) & NET_ACTION_PLAY)
	{
		if(NscStrPause())
		{
			(*stateflag) |= NET_ACTION_PAUSE;//turn pause bit on 
			(*stateflag) &= ~NET_ACTION_PLAY;//turn play bit off
			(*stateflag) &= ~NET_ACTION_STOP;//turn stop bit off
		}
		else
			return NETSTR_PAUSE_ERR;
	}
	//Play if already paused
	else if((*stateflag) & NET_ACTION_PAUSE)
	{
		if(NscStrPause())
		{
			(*stateflag) &= ~NET_ACTION_PAUSE;// turn pause bit off
			(*stateflag) &= ~NET_ACTION_STOP;// turn stop bit off
			(*stateflag) |= NET_ACTION_PLAY;// turn play bit on
		}
		else
			return NETSTR_RESUME_PLAY_ERR;
	}
	else
		return NETSTR_FLAG_MATCH_ERR;
	return 1;
} /* end of ProcessPause */

/**********************************************************************************
**              Syntax:         ProcessReverse(netflag_t *stateflag)             **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**		Returns:	1 for success					 **
**				error code in case of error			 **	
**                                                                               **
**              Comments:       None                                             **
**                                                                               **
**********************************************************************************/
static int
ProcessReverse(netflag_t *stateflag)
{
	//Reverse if in Play mode
	if((*stateflag) & NET_ACTION_PLAY)
	{
		if(NscStrPlayReverse())
		{
			(*stateflag) |= NET_ACTION_REVERSE;//turn rew bit on
			(*stateflag) &= ~NET_ACTION_PLAY;//turn play off
			(*stateflag) &= ~NET_ACTION_STOP;//turn stop off
		}
		else
			return NETSTR_REV_ERR;
	}
	//Play if already in reverse mode
	else if((*stateflag) & NET_ACTION_REVERSE)
	{
		if(NscStrPlay())
		{
			*stateflag &= ~NET_ACTION_REVERSE;//turn rew bit off
			*stateflag &= ~NET_ACTION_STOP;//turn stop bit off
			*stateflag |= NET_ACTION_PLAY;//turn play bit off
		}
		else	
			return NETSTR_RESUME_PLAY_ERR;
	}
	else
		return NETSTR_FLAG_MATCH_ERR;
	return 1;
} /* end of ProcessReverse */

/**********************************************************************************
**              Syntax:         ProcessFastForward(netflag_t *stateflag)         **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**              Returns:        1 for success                                    **
**                              error code in case of error                      **
**                                                                               **
**              Comments:       None                                             **
**                                                                               **
**********************************************************************************/
static int
ProcessFastForward(netflag_t *stateflag)
{
	//Forward if in Play mode
	if((*stateflag) & NET_ACTION_PLAY)
	{
		if(NscStrPlayFastForward())
		{
			*stateflag |= NET_ACTION_FORWARD;//turn fwd bit on
			*stateflag &= ~NET_ACTION_PLAY;//turn play bit off
			*stateflag &= ~NET_ACTION_STOP;//turn stop bit off
		}
		else
			return NETSTR_FFWD_ERR;
	}
	//Play if already in Forward mode
	else if((*stateflag) & NET_ACTION_FORWARD)
	{
		if(NscStrPlay())
		{
			*stateflag &= ~NET_ACTION_FORWARD;//turn fwd bit off
			*stateflag &= ~NET_ACTION_STOP;//turn stop fwd bit off
			*stateflag |= NET_ACTION_PLAY;//turn play bit on
		}
		else	
			return NETSTR_RESUME_PLAY_ERR;
	}
	else
		return NETSTR_FLAG_MATCH_ERR;
	return 1;
} /* end of ProcessFastForward */

/**********************************************************************************
**              Syntax:         ProcessMute(netflag_t *stateflag)                **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**              Returns:        1 for success                                    **
**                              error code in case of error                      **
**                                                                               **
**              Comments:       None                                             **
**                                                                               **
**********************************************************************************/
static int
ProcessMute(netflag_t *stateflag)
{
	//Mute if not in Mute mode
	if(!((*stateflag) & NET_ACTION_MUTE))
	{
		if(NscStrMute())
			(*stateflag) |= NET_ACTION_MUTE;
		else
			return NETSTR_MUTE_ERR;
	}
	//Restore volume if in mute mode
	else if((*stateflag) & NET_ACTION_MUTE)
	{
		if(NscStrMute())
			(*stateflag) &= ~NET_ACTION_MUTE;
		else
			return NETSTR_UNMUTE_ERR;
	}
	return 1;
} /* end of ProcessMute */

/**********************************************************************************
**              Syntax:         ProcessLoop(netflag_t *stateflag)                **
**                                                                               **
**              Parameters:     Pointer to stateflag of type short               **
**                                                                               **
**              Returns:        1 for success                                    **
**                              error code in case of error                      **
**                                                                               **
**                                                                               **
**              Comments:       None                                             **
**                                                                               **
**********************************************************************************/
static int
ProcessLoop(netflag_t *stateflag)
{
	//Loop if not in Loop mode
	if(!((*stateflag) & NET_ACTION_LOOP))
	{
		if(NscStrLoop())
			(*stateflag) |= NET_ACTION_LOOP;
		else
			return NETSTR_LOOP_ERR;
	}
	//Unloop if already in loop mode
	else if((*stateflag) & NET_ACTION_LOOP)
	{
		if(NscStrLoop())
			(*stateflag) &= ~NET_ACTION_LOOP;
		else
			return NETSTR_UNLOOP_ERR;
	}
	return 1;
} /* end of ProcessLoop */

/**********************************************************************************
**              Syntax:         ProcessSetLeftAudio(netflag_t *stateflag,int vol)**
**                                                                               **
**              Parameters:     Pointer to stateflag of type short,volume        **
**                                                                               **
**              Returns:        1 for success                                    **
**                              error code in case of error                      **
**                                                                               **
**              Comments:       None                                             **
**                                                                               **
**********************************************************************************/
static int
ProcessSetLeftAudio(netflag_t *stateflag,int vol)
{
	if(NscStrSetLeftAudio(vol) == 0)
		return NETSTR_LAUDIO_ERR;
	return 1;
}

/**********************************************************************************
**              Syntax:         ProcessSetRightAudio(netflag_t *stateflag,int vol**
**                                                                               **
**              Parameters:     Pointer to stateflag of type short,volume        **
**                                                                               **
**              Returns:        1 for success                                    **
**                              error code in case of error                      **
**                                                                               **
**              Comments:       None                                             **
**                                                                               **
**********************************************************************************/
static int
ProcessSetRightAudio(netflag_t *stateflag,int vol)
{
	if(NscStrSetRightAudio(vol) == 0)
		return NETSTR_RAUDIO_ERR;
	return 1;
}

int main(int argc, char *argv[])
{
	int	netrc,i;
 	int	vol;
	char	option;
	netflag_t stateflag = 0; //stores current state of the stream player
	int result; //to store resultof the various calls


	// recover the shared memory cmd_block pointer
	fprintf(stdout,"strplayer:DEBUG In wrapper application\n");
	if(clientMemoryAttach() != 0)
	{
		fprintf(stderr,"Unable to attach to shared memory...\n");
		exit(1); 
	 }
	else
		fprintf(stdout,"Client Mem Attached\n");

	// Initialise the device
	fprintf(stdout, "strplayer:DEBUG Opening the netstream device\n");
	if(NscStrInit() == 0)
		return NETSTR_INIT_ERR;
	

	// Get the initial command 
	while(!(stateflag & NET_EXIT))
	{
		netrc = 0;
		result = waitForCommand();
		switch (result)
		{

			// Connect to the MPEG server 
			case WMCMD_STR_CONNECT:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_INIT
				\n");
			//If already connected to server, do nothing
			if(stateflag & NET_ACTION_INIT)
				break;
			if((netrc = ProcessInit(&stateflag)) == 1)
				fprintf(stdout,"ProcessInit called\n");
			else
 			{
				fprintf(stderr,"ERROR: %s\n", printError(netrc));
				if((netrc = ProcessQuit(&stateflag)) != 1)
				{
					fprintf(stderr,"ERROR: %s\n",
							printError(netrc));
				}
			}  
			break;

			// Disconnect from the server 
			case WMCMD_STR_DISCONNECT:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_DISCONNECT\n");
			if((netrc = ProcessDisconnect(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Quit the application 
			case WMCMD_STR_QUIT:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_QUIT\n");
			if((netrc = ProcessQuit(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Play the MPEG Stream
			case WMCMD_STR_PLAY:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_PLAY\n");
			if((netrc = ProcessPlay(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Stop Playing the MPEG Stream
			case WMCMD_STR_STOP:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_STOP\n");
			if((netrc = ProcessStop(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Pause during Play
			case WMCMD_STR_PAUSE:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_PAUSE\n");
			if((netrc = ProcessPause(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Set Volume to Mute
			case WMCMD_STR_MUTE:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_MUTE\n");
			if((netrc = ProcessMute(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Fast Forward the Stream
			case WMCMD_STR_FWD:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_FWD\n");
			if((netrc = ProcessFastForward(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Reverse the Stream
			case WMCMD_STR_REV:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_REV\n");
			if((netrc = ProcessReverse(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Loop within the Stream
			case WMCMD_STR_LOOP:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_LOOP\n");
			if((netrc = ProcessLoop(&stateflag)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Control Volume of the Left Speaker
			case WMCMD_STR_LEFTAUDIO:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_LEFTAUDIO\n");
			if((netrc = ProcessSetLeftAudio(&stateflag,vol)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

			// Control Volume of the Right Speaker
			case WMCMD_STR_RIGHTAUDIO:
			fprintf(stdout, "strplayer:DEBUG Command = WMCMD_STR_RIGHTAUDIO\n");
			if((netrc = ProcessSetRightAudio(&stateflag,vol)) != 1)
				fprintf(stderr,"ERROR: %s\n",printError(netrc));
			break;

		}  
	} 
	freeMem();
	fprintf(stdout, "strplayer:DEBUG Closing API and exiting wrapper application\n");

}

