/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Netstream Plugin Files
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
*/
/******************************************************************************
*                                                                      	      *
*    			 FILENAME: netstrplugin.cpp                    	      * 
*			   National Semi Conductors			      *
*				   Bangalore				      *
*			 Authors: Lakshmi Gopinath, Chitra Shankaran	      *
*                                                                      	      *
******************************************************************************/


/*
**
** System "Include" Files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include <anlg_video.h>

#include "audio.h"
#include "cmdblock.h"
#include "cmdutil.h"
#include "dispatcher.h"
#include "errorbase.h"
#include "sharedclient.h"
#include "state.h"
#include "srcmpeg.h"

#define TIMEOUT 1
#define WRAPPER_LOCK_FILE	"/var/lock/netplayer.lock"

typedef struct
{
	char *cmd;
	int (*func)(int argc, char *argv[]);
}CmdEntry;

static char *plugin_name = " NetStream Player and Drive Support";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "netstr";

/**********************************************************************************
**                                                                               **
**		Syntax:		loadFrame(void)					 **
**										 **
**		Parameters:	None						 **
**										 **
**		Returns:	0						 **
**										 **
**		Comments:	None						 **
**										 **
**********************************************************************************/	
static int loadFrame(void)
{
	int tick;
	// make sure cmd_block is idle
	if(cmd_block->status != WMSTATUS_IDLE)
		return(ERRORBASE_STR + 1);

	// issue the command to load the html
	cmd_block->command = WMCMD_VIEWER_FRAME_LOAD;
	strcpy(cmd_block->parm1,"Dialog");
	printf("\n Setting cookie file dialognetstr.html\n");
	strcpy(cmd_block->parm2,"dialognetstr.html");
	cmd_block->status = WMSTATUS_START;
	
	// wait for the command to finish
	tick = time(NULL);
	while(cmd_block->status != WMSTATUS_DONE)
	{
		/*if(cmd_block->status == WMSTATUS_DONE)
	 		break;
	 	usleep(1000);
		*/
	    	if((tick + TIMEOUT) <= time(NULL))
	    	{
	      		cmd_block->command = WMCMD_NONE;
	      		cmd_block->status = WMSTATUS_IDLE;
	      		return(0);
	    	}
		
	  }
	  cmd_block->command = WMCMD_NONE;
	  cmd_block->status = WMSTATUS_IDLE;
  
	  return(0);
}

/*
**
** Command: source
**
**
**********************************************************************************
**		Syntax:		sourceFunc(int argc, char **argv)		**
**										**
**		Parameters:	None						**
**										**
**		Returns:	0						**
**										**
**										**
**		Comments:	Initialises video. Sets video input to MPEG	**
**				Decoder. Closes video with these attributes	**
**										**
*********************************************************************************/
static int sourceFunc(int argc, char *argv[])
{
	int	rc = 0;									
	int	tmp;

	if(VAPISrcMPEGSelect() != 0)
  	  	return(ERRORBASE_MPEG);

        VMW_AVS_VideoInit();
	VMW_OVL_EnableOverlay(0x00f000f0, 0x00f0f0f0, 0);
        rc = VIDEO_INPUT_MPEG_DECODER;
        VMW_AVS_SetCurrentVideoInput(&rc);
        VMW_AVS_VideoClose();
	SetAudio(AV_CHAN_MPEG,0,0);
	
	strcpy(state_source, plugin_keyword);
	loadFrame();
	fprintf(stdout,"Out of sourceFunc of netstr\n");
	
  	return(0);
}


/*
**
** Command: stop
**
**
**********************************************************************************
**		Syntax:		stopFunc(int argc, char **argv)			**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Stops the stream from playing			**
**										**
*********************************************************************************/
static int stopFunc(int argc, char *argv[])
{
  	printf("STR Stop function stub\n");
	return (WCMDU_ProcSimpleCmd(WMCMD_STR_STOP, ERRORBASE_STR + 2));
}


/*
**
** Command: init
**
**
**********************************************************************************
**		Syntax:		initFunc(int argc, char **argv)			**
**										**
**		Parameters:	None						**
**										**
**		Returns:	0 on success					**
**				errorcode on failure				**
**										**
**		Comments:	Initialises the stream 				**
**				execs the strplayer 				**
**				Updates the lockfile				**
**										**
*********************************************************************************/
static int initFunc(int argc, char *argv[])
{
	printf("STR init function stub\n");
  
	// Ensure that the wrapper application is running
	if (!WCMDU_WrAppRunning(WRAPPER_LOCK_FILE))
	{
		pid_t	pid;

	  	fprintf(stdout, "netstrplugin:DEBUG executing /bin/strplayer \n");
	  	if ( (pid = fork()) == 0)
	  	{
			// Child process
		  	execl("/bin/strplayer", "strplayer", "", NULL);
		  	return (ERRORBASE_STR + 3);
	  	} // end of if 

		WCMDU_UpdAppLckFile(pid, WRAPPER_LOCK_FILE);
		return (0);
	 } // end of if 
} // end of initFunc

/*
**
** Command: connect 
**
**
**********************************************************************************
**		Syntax:		connectFunc(int argc, char **argv)		**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Connects to the server				**
**										**
*********************************************************************************/
static int connectFunc(int argc, char *argv[])
{
	printf("STR play function stub\n");
	return (WCMDU_ProcSimpleCmd(WMCMD_STR_CONNECT, ERRORBASE_STR + 4));
}

/*
**
** Command: dinit
**
**
**********************************************************************************
**		Syntax:		dinitFunc(int argc, char **argv)		**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Disconnects from the server			**
**										**
*********************************************************************************/
static int dinitFunc(int argc, char *argv[])
{
	printf("STR play function stub\n");
	return (WCMDU_ProcSimpleCmd(WMCMD_STR_DISCONNECT, ERRORBASE_STR + 4));
}


/*
**
** Command: quit
**
**
**********************************************************************************
**		Syntax:		quitFunc(int argc, char **argv)			**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Quits the application				**
**										**
*********************************************************************************/
static int quitFunc(int argc, char *argv[])
{
	int rc;
	pid_t wpid; // Pid of wrapper app
        FILE *plckfile; // Lock file stream pointer
        printf("STR quit function stub\n");
	rc = (WCMDU_ProcSimpleCmd(WMCMD_STR_QUIT, ERRORBASE_STR + 4));

        // Open the lock file
        if ((plckfile = fopen(WRAPPER_LOCK_FILE, "r")) == NULL)
		printf("File Not present\n");
	else
	{
        	fscanf(plckfile, "%d", &wpid);
		if(!wpid)
        	kill(wpid,0);
		fclose(plckfile);
		remove(WRAPPER_LOCK_FILE);
		return rc;
	}
	return 0;
	
}


/*
**
** Command: play
**
**
**********************************************************************************
**		Syntax:		playFunc(int argc, char **argv)			**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Plays the stream				**
**										**
*********************************************************************************/
static int playFunc(int argc, char *argv[])
{
	printf("STR play function stub\n");
	return (WCMDU_ProcSimpleCmd(WMCMD_STR_PLAY, ERRORBASE_STR + 4));
}

/*
**
** Command: fast forward 
**
**
**********************************************************************************
**		Syntax:		ffwdFunc(int argc, char **argv)			**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Forwards the stream				**
**										**
*********************************************************************************/
static int ffwdFunc(int argc, char *argv[])
{
	printf("STR Fast Forward function stub\n");
	return (WCMDU_ProcSimpleCmd(WMCMD_STR_FWD, ERRORBASE_STR + 5));
}

/*
**
** Command: reverse 
**
**
**********************************************************************************
**		Syntax:		rewFunc(int argc, char **argv)			**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Reverses the stream				**
**										**
*********************************************************************************/
static int rewFunc(int argc, char *argv[])
{
	printf("STR Reverse function stub\n");
	return(WCMDU_ProcSimpleCmd(WMCMD_STR_REV, ERRORBASE_STR + 6));
}


/*
**
** Command: pause
**
**
**********************************************************************************
**		Syntax:		pauseFunc(int argc, char **argv)		**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Pauses the stream				**
**										**
*********************************************************************************/
static int pauseFunc(int argc, char *argv[])
{
	printf("STR Pause function stub\n");
	return(WCMDU_ProcSimpleCmd(WMCMD_STR_PAUSE, ERRORBASE_STR + 7));
}

/*
**
** Command: mute
**
**
**********************************************************************************
**		Syntax:		muteFunc(int argc, char **argv)			**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Plays the stream without volume			**
**										**
*********************************************************************************/
static int muteFunc(int argc, char *argv[])
{
	printf("STR Mute function stub\n");
	return(WCMDU_ProcSimpleCmd(WMCMD_STR_MUTE, ERRORBASE_STR + 8));
}

/*
**
** Command: loop
**
**
**********************************************************************************
**		Syntax:		loopFunc(int argc, char **argv)			**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Plays the stream in loop			**
**										**
*********************************************************************************/
static int loopFunc(int argc, char *argv[])
{
	printf("STR Loop function stub\n");
	return(WCMDU_ProcSimpleCmd(WMCMD_STR_LOOP, ERRORBASE_STR + 9));
}

/*
**
** Command: set left audio
**
**
**********************************************************************************
**		Syntax:		setleftaudioFunc(int argc, char **argv)		**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Sets the left audio				**
**										**
*********************************************************************************/
static int setleftaudioFunc(int argc, char *argv[])
{
	printf("STR SetLeftAudio function stub\n");
	return(WCMDU_ProcSimpleCmd(WMCMD_STR_LEFTAUDIO, ERRORBASE_STR + 10));
}

/*
**
** Command: setrightaudio
**
**
**********************************************************************************
**		Syntax:		setrightaudioFunc(int argc, char **argv)	**
**										**
**		Parameters:	None						**
**										**
**		Returns:	Value returned by function			**
**				WCMDU_ProcSimpleCmd()				**
**										**
**		Comments:	Sets the right audio				**
**										**
*********************************************************************************/
static int setrightaudioFunc(int argc, char *argv[])
{
	printf("STR SetRightAudio function stub\n");
	return(WCMDU_ProcSimpleCmd(WMCMD_STR_RIGHTAUDIO, ERRORBASE_STR + 11));
}

/*
**
** Command List
** Structure that conatins all the commands and the corresponding functions
**
*/
static CmdEntry cmdlist[] =
{
	"loop",loopFunc,
	"init",initFunc,
	"dinit",dinitFunc,
	"connect",connectFunc,
	"quit",quitFunc,
	"stop",stopFunc,
	"play",playFunc,
	"source",sourceFunc,
	"mute",muteFunc,
	"ffwd",ffwdFunc,
	"rew",rewFunc,
	"pause",pauseFunc,
	"setleftaudio",setleftaudioFunc,
	"setrightaudio",setrightaudioFunc,
	NULL,NULL
};

/*
**
** This function is called once, just after the plugin has been loaded
** and all plugin entrypoints have been registered. This function will
** be called before any other plugin entrypoint functions are called.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
	return(0);
}

/*
**
** This function is called once, just before the plugin is unloaded.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
	return(0);
}

/*
**
** This function is called to recover the plugin's name/description.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** name/description will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginName(char *result)
{
	strcpy(result,plugin_name);
	return(0);
}

/*
**
** This function is called to recover the plugin's internal version number.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** internal version number will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginVersion(char *result)
{
	strcpy(result,plugin_version);
	return(0);
}

/*
**
** This function is called to recover the plugin's command keyword.
**
** The "result" parameter is a pointer to the buffer where the plugin's
** command keyword will be stored.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
	strcpy(result,plugin_keyword);
	return(0);
}

/*
**
** This function is called to display this plugin's command set.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
	printf("   %s loop\n",plugin_keyword);
	printf("   %s stop\n",plugin_keyword);
	printf("   %s play\n",plugin_keyword);
	printf("   %s source\n",plugin_keyword);
	printf("   %s mute\n",plugin_keyword);
	printf("   %s ffwd\n",plugin_keyword);
	printf("   %s rew\n",plugin_keyword);
	printf("   %s pause\n", plugin_keyword);
	printf("   %s setleftaudio\n", plugin_keyword);
	printf("   %s setrightaudio\n", plugin_keyword);
	return(0);
}

/*
**
** This function is called to execute one of the plugin's commands.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a plugin entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
	int count,result;
	// scan the list of commands for a match
	count = 0;
	while(1)
	{
		// check for end of list (indicates unknown command)
		if(cmdlist[count].cmd == NULL)
		{
			result = ERRORBASE_STR + 0;
			break;
		}
		// check for command match
		if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
		{
			result = (cmdlist[count].func)(argc - 1,&argv[1]);
			break;
		}
		// bump to next command
		++count;
	}
	// return result and exit
	return(result);
}
