/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: OVERLAY.CPP                                                *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <sys/ioctl.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/videodev.h>
#include <linux/ioctl_geode_v.h>

#include "status.h"


/*
**
** Local Constant Definitions
**
*/



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/



/*
**
** Local Variable Declarations
**
*/


static int set_overlay_info(int xpos, int ypos, int width, int height, int enable_flag)
{
  int result,vhandle;

  overlay_info_t info;

  // open the output device
  vhandle = open("/dev/vout",O_RDWR);

  if(vhandle < 0)
  {
    VAPIStatusOutStr("Unable to open output device");
    return(-4);
  }

  // initialize OverlayInfo structure
  info.TypeOfData = 1;
  info.StartX = xpos & 0xfffffff8;
  info.StartY = ypos;
  info.Width = width & 0xfffffffe;
  info.Height = height;
  info.BufferOffset = 0xF0800;
  info.BufferWidth = width;
  info.BufferHeight = height;
  info.ColorKey = 0x00F000F0;
  info.ColorMask = 0x00F0F0F0;
  info.VideoFormat = 0;
  info.Graphics = 1;
  info.Enable = enable_flag;

  // output the OverlayInfo data
  result = ioctl(vhandle,GEODE_IOCTL_SET_OVERLAY_INFO,&info);

  close(vhandle);

  if(result < 0)
  {
    VAPIStatusOutStr("Unable to set overlay");
    //return(-5);
  }

  // issue status
  VAPIStatusOutStr("Overlay enabled");

  // exit with no errors
  return(0);
}

static int set_vout_window(int xpos, int ypos, int width, int height)
{
  int vhandle, result;
  struct v4l2_capability caps;
  struct v4l2_window wnd;

  // open the video device
  vhandle = open("/dev/vout",O_RDWR);

  if(vhandle < 0)
  {
    VAPIStatusOutStr("Unable to open video device");
    return(-1);
  }

  // initialize the v4l2_window structure
  memset(&wnd,0,sizeof(wnd));
  wnd.x = xpos & 0xfffffff8;
  wnd.y = ypos;
  wnd.width = width & 0xfffffffe;
  wnd.height = height;

  // output the OverlayInfo data
  result = ioctl(vhandle,VIDIOC_S_WIN,&wnd);

  close(vhandle);

  if(result < 0)
  {
    VAPIStatusOutStr("Overlay: unable to set window size");
    //return(-3);
  }

  return result;
}

static void preview(int enable)
{
  int result, vhandle;

  // open the video device
  vhandle = open("/dev/vout",O_RDWR);

  if(vhandle < 0)
  {
    VAPIStatusOutStr("Preview: Unable to open video device");
    return;
  }

  ioctl(vhandle, VIDIOC_PREVIEW,&enable);
  
  close(vhandle);
}

static void capture_enable( int enable)
{
  int result, vhandle;

  // open the video device
  vhandle = open("/dev/video",O_RDWR);

  if(vhandle < 0)
  {
    VAPIStatusOutStr("capture_enable: Unable to open video device");
    return;
  }

  if(enable)
    ioctl(vhandle, VIDIOC_STREAMON);
  else
    ioctl(vhandle, VIDIOC_STREAMOFF);

  close(vhandle);
}

static void set_video_fmt(int width, int height)
{
  int result, vhandle;
  struct v4l2_format fmt;

  memset(&fmt,0,sizeof(fmt));

  fmt.fmt.pix.width = width;
  fmt.fmt.pix.height = height;
  
  // open the video device
  vhandle = open("/dev/video",O_RDWR);

  if(vhandle < 0)
  {
    VAPIStatusOutStr("set_video_fmt: Unable to open video device");
    return;
  }

  result = ioctl(vhandle, VIDIOC_S_FMT, &fmt);

  close(vhandle);
}

/*
**
**
**
*/
static int setOverlay(int xpos,
		      int ypos,
		      int width,
		      int height,
		      int enable_flag)
{

  int result;

  set_video_fmt(width,height);

  capture_enable(1);

  result = set_vout_window(xpos,ypos,width,height);
  if(result < 0 )
    return result;

  result = set_overlay_info(xpos,ypos,width,height,enable_flag);
  if(result < 0)
    return(result);

  preview(1);  // turn on video

  return 0;
}



/*
**
** This function will initialize this module and prepare it for
** operation.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This funciton must be called before any other functions in this
** module are called.
**
*/
int VAPIOverlayOpen(void)
{
  // issue status
  VAPIStatusOutStr("Overlay opened");

  // exit with no errors
  return(0);
}



/*
**
** This function will close this module and return any used memory
** to the system.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPIOverlayClose(void)
{
  // issue status
  VAPIStatusOutStr("Overlay closed");

  // exit with no errors
  return(0);
}



/*
**
** This function will enable the video overlay operation.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPIOverlaySelect(void)
{
  return(setOverlay(0,0,720,486,1));
}



/*
**
** This function will disable the video overlay operation.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPIOverlayDisable(void)
{
  return(setOverlay(0,0,720,486,0));
}



/*
**
** This function will position the overlay window on the screen at
** the specified position and size.
**
** The "xpos" parameter specifies the horizontal position of the 
** overlay window. The "ypos" parameter specifies the vertical
** position of the overlay window. The "width" parmater specifies
** the width of the overlay window. The "height" paramter specifies
** the height of the overlay window.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPIOverlayPosition(int xpos, int ypos, int width, int height)
{
  return(setOverlay(xpos,ypos,width,height,1));
}



/*
**
**
**
*/
int debugOverlay(int winX,
		 int winY,
		 int winW,
		 int winH,
		 int ovrX,
		 int ovrY,
		 int ovrW,
		 int ovrH,
		 int bufW,
		 int bufH)
{
  int buf[256],result,vhandle;
  overlay_info_t *pinfo;
  struct v4l2_window wnd;

  // open the video device
  vhandle = open("/dev/video",O_RDONLY);
  if(vhandle < 0)
    return(-1);

  // initialize the v4l2_window structure
  memset(&wnd,0,sizeof(wnd));
  wnd.x = winX;
  wnd.y = winY;
  wnd.width = winW;
  wnd.height = winH;

  // output the OverlayInfo data
  result = ioctl(vhandle,VIDIOC_S_WIN,&wnd);
  close(vhandle);
  if(result < 0)
    return(-2);

  // open the output device
  vhandle = open("/dev/vout",O_WRONLY);
  if(vhandle < 0)
    return(-3);

  // initialize OverlayInfo structure
  pinfo = (overlay_info_t *)buf;
  pinfo->TypeOfData = 1;
  pinfo->StartX = ovrX;
  pinfo->StartY = ovrY;
  pinfo->Width = ovrW;
  pinfo->Height = ovrH;
  pinfo->BufferOffset = 0xF0800;
  pinfo->BufferWidth = bufW;
  pinfo->BufferHeight = bufH;
  pinfo->ColorKey = 0x00F000F0;
  pinfo->ColorMask = 0x00F0F0F0;
  pinfo->VideoFormat = 0;
  pinfo->Graphics = 1;
  pinfo->Enable = 1;

  // output the OverlayInfo data
  result = ioctl(vhandle,GEODE_IOCTL_SET_OVERLAY_INFO,pinfo);
  close(vhandle);

  // exit with no errors
  return(0);
}





