/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: sharedserver.                                              *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include "cmdblock.h"
#include "state.h"



/*
**
** Global Variable Declarations
**
*/
WMCmdBlock *cmd_block;



/*
**
** Local Constant Definitions
**
*/



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/



/*
**
** Local Variable Declarations
**
*/



/*
**
** This function will recover a pointer to the shared memory block
** allocated for WebMedia applications. If the shared memory block
** does not exist, one is created.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int serverMemoryAttach(void)
{
  int id;

  // clear the current pointer
  cmd_block = NULL;

  // attempt to attach to an existing block
  cmd_block = (WMCmdBlock *)shmat(state_sharedmem_id,0,0);
  if(cmd_block != ((WMCmdBlock *)-1))
    return(0);

  // block does not exist - create one
  id = shmget(IPC_PRIVATE,sizeof(WMCmdBlock),0777);
  if(id == -1)
    return(-1);
  state_sharedmem_id = id;

  // block created - now attach to it
  cmd_block = (WMCmdBlock *)shmat(state_sharedmem_id,0,0);
  if(cmd_block == ((WMCmdBlock *)-1))
    return(-2);

  // setup initial block values
  cmd_block->status = WMSTATUS_IDLE;
  cmd_block->command = WMCMD_NONE;
  cmd_block->parm1[0] = 0x00;
  cmd_block->parm2[0] = 0x00;
  cmd_block->parm3 = 0;
  cmd_block->parm4 = 0;

  // exit with no errors
  return(0);
}



/*
**
** This function will free the memory allocated from a previous call
** to the "sharedMemoryAttach()" function.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int serverMemoryDetach(void)
{
  if(cmd_block != NULL)
  {
    shmdt(cmd_block);
    cmd_block = NULL;
  }
}
