/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: sourceplugin.cpp                                           *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "cmdutil.h"
#include "cmdblock.h"
#include "dispatcher.h"
#include "errorbase.h"
#include "state.h"



/*
**
** Local Constant Definitions
**
*/
#define DVD_LOCK_FILE "/var/lock/dvdplayer.lock"
#define MPEG_LOCK_FILE "/var/lock/mpeg.lock"

/* Added netstr lock - NIDC */
#define NETSTR_LOCK_FILE "/var/lock/netplayer.lock"


/*
**
** Local Enumeration Definitions
**
*/
static char *plugin_name = "Multisource Sequential Switching";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "source";



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char cmdbuf[256];

enum sourcetype_t
{	STRNG_CVIDEO,
	STRNG_SVIDEO,
	STRNG_TVTUNER,
	STRNG_MPEG,
	STRNG_NETSTR,
	STRNG_DVD
	}; // to get the source type


/*
**
** compareString
**
*/
static sourcetype_t compareString()
{
	if(!strcmp(state_source,"cvideo"))
		return STRNG_CVIDEO;
	if(!strcmp(state_source,"svideo"))
		return STRNG_SVIDEO;
	if(!strcmp(state_source,"tuner"))
		return STRNG_TVTUNER;
	if(!strcmp(state_source,"netstr"))
		return STRNG_NETSTR;
	if(!strcmp(state_source,"mpeg"))
		return STRNG_MPEG;
	if(!strcmp(state_source,"dvd"))
		return STRNG_DVD;
}


/*
**
** Command: step
**
*/
static int stepFunc(int argc, char *argv[])
{
	char	altbuf[256] = {0};
	int	rc;
	sourcetype_t source_used;
	
	printf("In Step Func\n");
	source_used = compareString();	
	switch(source_used)
	{
		case STRNG_CVIDEO:
  		{
			fprintf(stdout, "shifting to mpeg \n");
			strcpy(cmdbuf,"mpeg source");
			break;
 		 }	 
		case STRNG_MPEG:
  		{
			//is mpeg running? if yes close it
			if (WCMDU_WrAppRunning(MPEG_LOCK_FILE))
			{
				printf("Mpeg process running, issuing a WMCMD_MPEG_STOP command.\n");
				strcpy(altbuf, "mpeg stop");
			}
			fprintf(stdout, "shifting to netstr\n");
   			strcpy(cmdbuf,"netstr source");
			break;
  		} 
		case STRNG_NETSTR:
  		{
			//is netstr running? if yes close it
			if (WCMDU_WrAppRunning(NETSTR_LOCK_FILE))
			{
				printf("NetStr process running, issuing a WMCMD_STR_STOP command.\n");
				strcpy(altbuf, "netstr quit");
			}	 
			fprintf(stdout, "shifting to svideo\n");
			strcpy(cmdbuf,"svideo source");
			break;

		}
		case STRNG_SVIDEO: 	
  		{
		    	fprintf(stdout, "shifting to tuner\n");
			strcpy(cmdbuf,"tuner source");
			break;
  		} 
		case STRNG_TVTUNER:
  		{
			fprintf(stdout, "shifting to dvd\n");
			strcpy(cmdbuf, "dvd source");
			break;
  		} 
		case STRNG_DVD:
  		{
			//is dvd running? if yes close it
		  	if (WCMDU_WrAppRunning(DVD_LOCK_FILE))
  			{
				printf("DVD process running, issuing a WMCMD_DVD_STOP command.\n");
				strcpy(altbuf, "dvd stop");
  			} 
			fprintf(stdout, "shifting to cvideo\n");
		  	strcpy(cmdbuf, "cvideo source");
			break;
  		} 
	} /*end of switch*/

	printf("Step func for %s\n",cmdbuf);

	rc = dispatcher(cmdbuf);
	if (*altbuf)
		dispatcher(altbuf);

	return (rc);
}


/*
**
** Command: info
**
*/
static int infoFunc(int argc, char *argv[])
{
  printf("Source: %s\n",state_source);
  return(0);
}



/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "step",stepFunc,
  "info",infoFunc,
  NULL,NULL
};



/*
**
** This function will initialize the source module and prepare it
** for subsequent operations.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function will closeout the source module and shut it down.
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function will return the subsystem identification string
** for the source module ("source").
**
** The "result" parameter is a pointer to the buufer where the
** subsystem identification string will be stored.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
** This function will display the commands available for the source
** module.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s step\n",plugin_keyword);
  printf("   %s info\n",plugin_keyword);
  return(0);
}



/*
**
** This function will execute the specified source module command.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result;

  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_SOURCE + 0;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

  // return result and exit
  return(result);
}
