/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: SRCTUNER.CPP                                               *
*                                                                      *
***********************************************************************/


/*
**
** Imported "Include" Files
**
*/
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <linux/fs.h>
#include <linux/videodev.h>
#include "audio.h"
#include "srctuner.h"
#include "status.h"

#include <linux/soundcard.h>

/*
**
** Local Constant Definitions
**
*/
#define VIDEO_DEVICE "/dev/video"

/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/



/*
**
** Local Variable Declarations
**
*/


/*
**
** This function will select the tuner source as the system video
** source.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPISrcTunerSelect(void)
{

#ifndef VAPI_STUB

  char buf[256];
  int result,source,vhandle;

  // open the video device
  vhandle = open(VIDEO_DEVICE,O_RDONLY);
  if(vhandle < 0)
  {
    sprintf(buf,"no video device: %s",VIDEO_DEVICE);
    VAPIStatusOutStr(buf);
    return(-1);
  }

  // Set the audio correctly
  source = AV_CHAN_TUNER;
  SetAudio(source,0,0);

  // select the tuner video source
//  result = ioctl(vhandle,VIDIOC_S_INPUT,&source);
//  if(result < 0)
//  {
//    VAPIStatusOutStr("unable to select tuner source");
//    return(result);
//  }
  close(vhandle);

  // issue status
  VAPIStatusOutStr("tuner video source selected");

#endif

  // exit with no errors
  return(0);
}



/*
**
** This function will set the tuner to the specified channel.
**
** The "channel" parameter specifies the tuner channel.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPISrcTunerChannelSet(int channel)
{

#ifndef VAPI_STUB

  FILE *fp;
  char buf[256];
  int result,vhandle;

  // open the video device
  vhandle = open(VIDEO_DEVICE,O_RDONLY);
  if(vhandle < 0)
  {
    sprintf(buf,"no video device: %s",VIDEO_DEVICE);
    VAPIStatusOutStr(buf);
    return(-1);
  }

  // set the channel
  result = ioctl(vhandle,VIDIOC_S_FREQ,&channel);
  if(result < 0)
  {
    sprintf(buf,"unable to set tuner to channel %d",channel);
    VAPIStatusOutStr(buf);
    return(result);
  }

  close(vhandle);

  // issue status
  sprintf(buf,"tuner set to channel %d",channel);
  VAPIStatusOutStr(buf);

#endif

  // exit with no errors
  return(0);
}














