/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: state.cpp                                                  *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include "errorbase.h"
#include "rsctools.h"
#include "state.h"



/*
**
** Global Variable Declarations
**
*/
char state_source[256];
int state_channel;
int state_channel_max;
int state_channel_min;
int state_vidwin_xpos;
int state_vidwin_ypos;
int state_vidwin_width;
int state_vidwin_height;
char state_html_base[256];
int state_volumes[4];
idbugdata_t state_idbug[3] = {0};


/*
**
** Local Constant Definitions
**
*/
#define STATE_PATHNAME "/tmp/.wmcmd.dat"



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/



/*
**
** Local Variable Declarations
**
*/



/*
**
** This function will load and parse the state textfile and extract
** all global variables.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int loadState(void)
{
  FILE *fp;
  char *ptr[2];

  // set default state values
  strcpy(state_source,"tuner");
  state_channel = 2;
  state_channel_max = 181;
  state_channel_min = 2;
  state_vidwin_xpos = 0;
  state_vidwin_ypos = 0;
  state_vidwin_width = 640;
  state_vidwin_height = 480;
 // strcpy(state_html_base,"/usr/nanox/html");
strcpy(state_html_base,"/var/html");

  // Default volume data taken from Phat's vtest.cc
  state_volumes[0] = 15;
  state_volumes[1] = 95;
  state_volumes[2] = 95;
  state_volumes[3] = 30;
  state_idbug[0].xPos = state_idbug[0].yPos = 0;
  state_idbug[0].transLvl = (int)(255 / 2);
  state_idbug[0].pid = 0;
  state_idbug[0].idFile[0] = '\0';
  state_idbug[1].xPos = state_idbug[1].yPos = 0;
  state_idbug[1].transLvl = (int)(255 / 2);
  state_idbug[1].pid = 0;
  state_idbug[1].idFile[0] = '\0';
  state_idbug[2].xPos = state_idbug[2].yPos = 0;
  state_idbug[2].transLvl = (int)(255 / 2);
  state_idbug[2].pid = 0;
  state_idbug[2].idFile[0] = '\0';

  // open the state file
  if((fp = fopen(STATE_PATHNAME,"r")) == NULL)
    return(ERRORBASE_STATE + 0);

  // load the CURRENT_SOURCE information
  ptr[0] = state_source;
  if(locateKeyword(fp,"SOURCE") <= 0)
    return(ERRORBASE_STATE + 1);
  if(getStringParm(fp,ptr,1) != 1)
    return(ERRORBASE_STATE + 2);

  // load the CHANNEL information
  if(locateKeyword(fp,"CHANNEL") <= 0)
    return(ERRORBASE_STATE + 3);
  if(getIntegerParm(fp,&state_channel,1) != 1)
    return(ERRORBASE_STATE + 4);

  // load the CHANNEL_MAX information
  if(locateKeyword(fp,"CHANNEL_MAX") <= 0)
    return(ERRORBASE_STATE + 5);
  if(getIntegerParm(fp,&state_channel_max,1) != 1)
    return(ERRORBASE_STATE + 6);

  // load the CHANNEL_MIN information
  if(locateKeyword(fp,"CHANNEL_MIN") <= 0)
    return(ERRORBASE_STATE + 7);
  if(getIntegerParm(fp,&state_channel_min,1) != 1)
    return(ERRORBASE_STATE + 8);

  // load the VIDWIN_XPOS information
  if(locateKeyword(fp,"VIDWIN_XPOS") <= 0)
    return(ERRORBASE_STATE + 9);
  if(getIntegerParm(fp,&state_vidwin_xpos,1) != 1)
    return(ERRORBASE_STATE + 10);

  // load the VIDWIN_YPOS information
  if(locateKeyword(fp,"VIDWIN_YPOS") <= 0)
    return(ERRORBASE_STATE + 11);
  if(getIntegerParm(fp,&state_vidwin_ypos,1) != 1)
    return(ERRORBASE_STATE + 12);

  // load the VIDWIN_WIDTH information
  if(locateKeyword(fp,"VIDWIN_WIDTH") <= 0)
    return(ERRORBASE_STATE + 13);
  if(getIntegerParm(fp,&state_vidwin_width,1) != 1)
    return(ERRORBASE_STATE + 14);

  // load the VIDWIN_HEIGHT information
  if(locateKeyword(fp,"VIDWIN_HEIGHT") <= 0)
    return(ERRORBASE_STATE + 15);
  if(getIntegerParm(fp,&state_vidwin_height,1) != 1)
    return(ERRORBASE_STATE + 16);

  // load the HTML_BASE information
  ptr[0] = state_html_base;
  if(locateKeyword(fp,"HTML_BASE") <= 0)
    return(ERRORBASE_STATE + 17);
  if(getStringParm(fp,ptr,1) != 1)
    return(ERRORBASE_STATE + 18);
  
  // Load the VOLUME for channel 0 information
  if (locateKeyword(fp, "VOLUME0") <= 0)
	  return (ERRORBASE_STATE + 19);
  if (getIntegerParm(fp, &state_volumes[0], 1) != 1)
  	return (ERRORBASE_STATE + 20);
  
  // Load the VOLUME for channel 1 information
  if (locateKeyword(fp, "VOLUME1") <= 0)
	  return (ERRORBASE_STATE + 21);
  if (getIntegerParm(fp, &state_volumes[1], 1) != 1)
  	return (ERRORBASE_STATE + 22);

  // Load the VOLUME for channel 2 information
  if (locateKeyword(fp, "VOLUME2") <= 0)
	  return (ERRORBASE_STATE + 23);
  if (getIntegerParm(fp, &state_volumes[2], 1) != 1)
	  return (ERRORBASE_STATE + 24);

  // Load the VOLUME for channel 3 information
  if (locateKeyword(fp, "VOLUME3") <= 0)
	  return (ERRORBASE_STATE + 25);
  if (getIntegerParm(fp, &state_volumes[3], 1) != 1)
	  return (ERRORBASE_STATE + 26);

  // Load the IDBUG state values for alpha channel 1
  if (locateKeyword(fp, "IDBUG1_XPOS") > 0)
	  if (getIntegerParm(fp, &state_idbug[0].xPos, 1) != 1)
		  return (ERRORBASE_STATE + 27);
  if (locateKeyword(fp, "IDBUG1_YPOS") > 0)
	  if (getIntegerParm(fp, &state_idbug[0].yPos, 1) != 1)
		  return (ERRORBASE_STATE + 28);
  if (locateKeyword(fp, "IDBUG1_LVL") > 0)
	  if (getIntegerParm(fp, &state_idbug[0].transLvl, 1) != 1)
		  return (ERRORBASE_STATE + 29);
  if (locateKeyword(fp, "IDBUG1_PID") > 0)
	  if (getIntegerParm(fp, &state_idbug[0].pid, 1) != 1)
		  return (ERRORBASE_STATE + 30);
  ptr[0] = state_idbug[0].idFile;
  if (locateKeyword(fp, "IDBUG1_IMAGE") > 0)
	  getStringParm(fp,ptr,1);

  // Load the IDBUG state values for alpha channel 2
  if (locateKeyword(fp, "IDBUG2_XPOS") > 0)
	  if (getIntegerParm(fp, &state_idbug[1].xPos, 1) != 1)
		  return (ERRORBASE_STATE + 32);
  if (locateKeyword(fp, "IDBUG2_YPOS") > 0)
	  if (getIntegerParm(fp, &state_idbug[1].yPos, 1) != 1)
		  return (ERRORBASE_STATE + 33);
  if (locateKeyword(fp, "IDBUG2_LVL") > 0)
	  if (getIntegerParm(fp, &state_idbug[1].transLvl, 1) != 1)
		  return (ERRORBASE_STATE + 34);
  if (locateKeyword(fp, "IDBUG2_PID") > 0)
	  if (getIntegerParm(fp, &state_idbug[1].pid, 1) != 1)
		  return (ERRORBASE_STATE + 35);
  ptr[0] = state_idbug[1].idFile;
  if (locateKeyword(fp, "IDBUG2_IMAGE") > 0)
	 getStringParm(fp,ptr,1);

  // Load the IDBUG state values for alpha channel 3
  if (locateKeyword(fp, "IDBUG3_XPOS") > 0)
	  if (getIntegerParm(fp, &state_idbug[2].xPos, 1) != 1)
		  return (ERRORBASE_STATE + 37);
  if (locateKeyword(fp, "IDBUG3_YPOS") > 0)
	  if (getIntegerParm(fp, &state_idbug[2].yPos, 1) != 1)
		  return (ERRORBASE_STATE + 38);
  if (locateKeyword(fp, "IDBUG3_LVL") > 0)
	  if (getIntegerParm(fp, &state_idbug[2].transLvl, 1) != 1)
		  return (ERRORBASE_STATE + 39);
  if (locateKeyword(fp, "IDBUG3_PID") > 0)
	  if (getIntegerParm(fp, &state_idbug[2].pid, 1) != 1)
		  return (ERRORBASE_STATE + 40);
  ptr[0] = state_idbug[2].idFile;
  if (locateKeyword(fp, "IDBUG3_IMAGE") > 0)
	  getStringParm(fp,ptr,1);

  // close the state file
  fclose(fp);

  // exit with no errors
  return(0);
}



/*
**
** This function will save the current global variable set as the
** state textfile.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int saveState(void)
{
  FILE *fp;

  // open the file
  if((fp = fopen(STATE_PATHNAME,"w")) == NULL)
    return(ERRORBASE_STATE + 5);

  // write the header information
  fprintf(fp,"*****************************\n");
  fprintf(fp,"* Century Software          *\n");
  fprintf(fp,"* Current State Information *\n");
  fprintf(fp,"*****************************\n\n");

  // write the state information
  fprintf(fp,"SOURCE: %s\n",state_source);
  fprintf(fp,"CHANNEL: %d\n",state_channel);
  fprintf(fp,"CHANNEL_MAX: %d\n",state_channel_max);
  fprintf(fp,"CHANNEL_MIN: %d\n",state_channel_min);
  fprintf(fp,"VIDWIN_XPOS: %d\n",state_vidwin_xpos);
  fprintf(fp,"VIDWIN_YPOS: %d\n",state_vidwin_ypos);
  fprintf(fp,"VIDWIN_WIDTH: %d\n",state_vidwin_width);
  fprintf(fp,"VIDWIN_HEIGHT: %d\n",state_vidwin_height);
  fprintf(fp,"HTML_BASE: %s\n",state_html_base);
  fprintf(fp, "VOLUME0: %d\n", state_volumes[0]);
  fprintf(fp, "VOLUME1: %d\n", state_volumes[1]);
  fprintf(fp, "VOLUME2: %d\n", state_volumes[2]);
  fprintf(fp, "VOLUME3: %d\n", state_volumes[3]);
  fprintf(fp, "IDBUG1_XPOS: %d\n", state_idbug[0].xPos);
  fprintf(fp, "IDBUG1_YPOS: %d\n", state_idbug[0].yPos);
  fprintf(fp, "IDBUG1_LVL: %d\n", state_idbug[0].transLvl);
  fprintf(fp, "IDBUG1_PID: %d\n", state_idbug[0].pid);
  fprintf(fp, "IDBUG1_IMAGE: %s\n", state_idbug[0].idFile);
  fprintf(fp, "IDBUG2_XPOS: %d\n", state_idbug[1].xPos);
  fprintf(fp, "IDBUG2_YPOS: %d\n", state_idbug[1].yPos);
  fprintf(fp, "IDBUG2_LVL: %d\n", state_idbug[1].transLvl);
  fprintf(fp, "IDBUG2_PID: %d\n", state_idbug[1].pid);
  fprintf(fp, "IDBUG2_IMAGE: %s\n", state_idbug[1].idFile);
  fprintf(fp, "IDBUG3_XPOS: %d\n", state_idbug[2].xPos);
  fprintf(fp, "IDBUG3_YPOS: %d\n", state_idbug[2].yPos);
  fprintf(fp, "IDBUG3_LVL: %d\n", state_idbug[2].transLvl);
  fprintf(fp, "IDBUG3_PID: %d\n", state_idbug[2].pid);
  fprintf(fp, "IDBUG3_IMAGE: %s\n", state_idbug[2].idFile);

  // close the file
  fclose(fp);

  // exit with no errors
  return(0);
}

