/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: STATUS.CPP                                                 *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <stdio.h>
#include "status.h"



/*
**
** Local Constant Definitions
**
*/
#define LOGNAME "video.log"



/*
**
** Local Variable Declarations
**
*/
static int curmode;



/*
**
** This function will open and initialize this module.
**
** The "mode" parameter specifies how the status text will be
** displayed, and is selected from the StatusMode enumeration.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function must be called before any other functions in this
** module are called.
**
*/
int VAPIStatusOpen(int mode)
{
  FILE *fp;

  // save the specified mode
  curmode = mode;

  // add log header
  if(curmode & STATUSMODE_FILE)
  {
    if((fp = fopen(LOGNAME,"a+")) == NULL)
      return(-1);
    fprintf(fp,"\n*** Status Logging Enabled ***\n\n");
    fclose(fp);
  }

  // exit with no errors
  return(0);
}



/*
**
** This function will close this module and return any allocated
** memory back to the system.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPIStatusClose(void)
{
  // exit with no errors
  return(0);
}




/*
**
** This function will output the specified string to all active
** status devices.
**
** The "str" parameter is a pointer to a buffer that contains the
** text to be output.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPIStatusOutStr(char *str)
{
  FILE *fp;

  // check for screen output
  if(curmode & STATUSMODE_SCREEN)
    printf("VAPI: %s\n",str);

  // check for log output
  if(curmode & STATUSMODE_FILE)
  {
    if((fp = fopen(LOGNAME,"a+")) == NULL)
      return(-1);
    fprintf(fp,"%s\n",str);
    fclose(fp);
  }

  // exit with no errors
  return(0);
}



/*
**
** This function will output the specified value to all active
** status devices.
**
** The "value" parameter contains the value to be output.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
*/
int VAPIStatusOutVal(int value)
{
  FILE *fp;

  // check for screen output
  if(curmode & STATUSMODE_SCREEN)
    printf("VAPI: %d\n",value);

  // check for log output
  if(curmode & STATUSMODE_FILE)
  {
    if((fp = fopen(LOGNAME,"a+")) == NULL)
      return(-1);
    fprintf(fp,"%d\n",value);
    fclose(fp);
  }

  // exit with no errors
  return(0);
}
