/********************************************************************************
    Copyright (C) 1999  Dirk Farin
    Copyright (C) 2001  Century Embedded Technologies

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#include "streamsrc_linux_http.hh"

#include <math.h>
#include <iostream.h>
#include <string.h>


bool StreamSource_HTTP::writestring(int fd, char *string)
{

  int result,bytes=strlen(string);

  while(bytes)
  {
    if((result=write(fd,string,bytes))<0 && errno!=EINTR)
    {
      cerr << "HTTP_ERROR_HTTPWRITEFAIL(writestring1)\n";
      return false;
    }
    else if(result==0)
    {
      cerr << "HTTP_ERROR_HTTPWRITEFAIL(writestring2)\n";
      return false;
    }
    string += result;
    bytes -= result;
  }

  return true;
}

bool StreamSource_HTTP::readstring(char *string,int maxlen,FILE *f)
{

  char *result;

  do{
    result=fgets(string,maxlen,f);
  }while(!result  && errno==EINTR);
  if(!result)
  {
    cerr << "HTTP_ERROR_FILEREADFAIL(readstring)\n";
    return false;
  }

  return true;
}

static char *strndup(char *src,int num)
{
  char *dst;

  if(!(dst=new char[num+1]))return NULL;
  dst[num]='\0';

  return strncpy(dst, src, num);
}

static char *url2hostport(char *url,char **hname,
			  unsigned long *hip,unsigned int *port)
{

  char *cptr;
  struct hostent *myhostent;
  struct in_addr myaddr;
  int isip=1;

  if(!(strncmp(url,httpstr,7)))url+=7;
  cptr=url;
  while(*cptr && *cptr!=':' && *cptr!='/')
  {
    if((*cptr<'0' || *cptr>'9') && *cptr!='.')isip=0;
    cptr++;
  }
  if(!(*hname=strndup(url,cptr-url)))
  {
    *hname=NULL;
    return NULL;
  }
  if(!isip)
  {
    if (!(myhostent=gethostbyname(*hname)))return NULL;
    memcpy(&myaddr,myhostent->h_addr,sizeof(myaddr));
    *hip=myaddr.s_addr;
  }
  else if((*hip=inet_addr(*hname))==INADDR_NONE)return NULL;
  if(!*cptr || *cptr=='/')
  {
    *port=80;
    return cptr;
  }
  *port=atoi(++cptr);
  while(*cptr && *cptr!='/')cptr++;
  return cptr;
}

StreamSource_HTTP::StreamSource_HTTP()
  : fp(0), d_buflen(0)
{
}


StreamSource_HTTP::~StreamSource_HTTP()
{
  if(fp)fclose(fp);
}

void StreamSource_HTTP::Init(const char* url)
{
  if((fp=http_open(url))==NULL) {
    cerr << "COULD NOT OPEN URL: " << url << endl;
    exit(-1);
  }
}

char *proxyurl=NULL;
unsigned long proxyip=0;
unsigned int proxyport;

FILE *StreamSource_HTTP::http_open(char *url) {

  char *purl=NULL,*host,*request,*sptr;
  char agent[50];
  int linelength;
  unsigned long myip;
  unsigned int myport;
  int sock;
  int relocate=0,numrelocs=0;
  struct sockaddr_in server;
  FILE *myfile;

  if(!proxyip)
  {
    if(!proxyurl)
      if(!(proxyurl=getenv("MPEG_HTTP_PROXY")))
	if(!(proxyurl=getenv("http_proxy")))
	  proxyurl = getenv("HTTP_PROXY");
    if (proxyurl && proxyurl[0] && strcmp(proxyurl, "none"))
    {
      if (!(url2hostport(proxyurl, &host, &proxyip, &proxyport)))
      {
	cerr << "HTTP_ERROR_UNKNOWNPROXY(http_open1)\n";
	return NULL;
      }
      if(host)
	delete host;
    }
    else
      proxyip = INADDR_NONE;
  }
  
  if((linelength=strlen(url)+100)<1024)
    linelength=1024; 
  if(!(request = new char[1024]) || !(purl = new char[1024]) )
  {
    cerr << "HTTP_ERROR_MEMORYNOTENOUGH(http_open2)\n";
    return NULL;
  }
  strncpy(purl,url,1023);
  purl[1023]='\0';
  do{
    strcpy(request,"GET ");
    if(proxyip!=INADDR_NONE) 
    {
      if(strncmp(url,httpstr,7))
	strcat(request,httpstr);
      strcat(request,purl);
      myport=proxyport;
      myip=proxyip;
    }
    else
    {
      if(!(sptr=url2hostport(purl,&host,&myip,&myport)))
      {
	cerr << "HTTP_ERROR_UNKNOWNHOST(http_open2)\n";
	return NULL;
      }
      if (host)
	delete (host);
      strcat (request, sptr);
    }
    sprintf (agent, " HTTP/1.0\r\nUser-Agent: %s/%s\r\n\r\n",
	     "dvdvideo","1.1.0");
    strcat (request, agent);
    server.sin_family = AF_INET;
    server.sin_port = htons(myport);
    server.sin_addr.s_addr = myip;
    
    if((sock=socket(PF_INET,SOCK_STREAM,6))<0)
    {
      cerr << "HTTP_ERROR_SOCKET(http_open4)\n";
      return NULL;
    }

    if(connect(sock,(struct sockaddr *)&server,sizeof(server)))
    {
      cerr << "HTTP_ERROR_CONNECT(http_open5)\n";
      return NULL;
    }
    if(!writestring(sock,request))return NULL;
    
    if(!(myfile=fdopen(sock, "rb")))
    {
      cerr << "HTTP_ERROR_FDOPEN(http_open6)\n";
      return NULL;
    };
    relocate=false;
    purl[0]='\0';
    if(!readstring(request,linelength-1,myfile))return NULL;
    
    if((sptr=strchr(request,' ')))
    {
      switch(sptr[1])
      {
        case '3':relocate=true;
        case '2':break;
      default: cerr << "HTTP_ERROR_HTTPFAIL(http_open7)\n";
	         return NULL;
      }
    }
    do{
      if(!readstring(request,linelength-1,myfile))return NULL;
      if(!strncmp(request,"Location:",9))
	strncpy (purl,request+10,1023);
    }while(request[0]!='\r' && request[0]!='n');
  }while(relocate && purl[0] && numrelocs++<5);
  if(relocate)
  { 
    cerr << "HTTP_ERROR_TOOMANYRELOC(http_open8)\n";
    return NULL;
  }
  delete[] purl;
  delete[] request;
  return myfile;
} 

void dump(uint8* mem, int size) {
  for ( int i=0; i < size; i++ ) {
    cerr << hex << (int)mem[i] << " ";
  }
}

int StreamSource_HTTP::FillBuffer(BYTE* mem, DWORD maxlen)
{
  //cerr << "FillBuffer(" << maxlen << ")\n";
  int bytesread=0;

  if (!fp)
    return bytesread;

  if (d_buflen!=0)
    {
      int size = min(maxlen,d_buflen);
      char *s_buf = new char[size];
      bytesread=getblock(s_buf, size);
      //if (!bytesread)
	//return (-1);
      memcpy(mem, s_buf, size); 
      d_buflen -= size;
      d_bufidx += size;
      delete[] s_buf;
      return size;
    }
  else
    {
      if (maxlen>=32768)
        {
	  char *s_buf = new char[32768];
          bytesread=getblock(s_buf,32768);
	  //if (!bytesread)
	    //return (-1);
	  memcpy(mem, s_buf, 32768); 
	  delete[] s_buf;
          return 32768;
        }
      else
        {
	  char *s_buf = new char[maxlen];
	  bytesread=getblock(s_buf, maxlen);
	  //if (!bytesread)
	    //return (-1);
	  memcpy(mem, s_buf, maxlen); 
	  //dump(mem, maxlen);
          d_bufidx = maxlen;
          d_buflen = 2324-maxlen;
	  delete[] s_buf;
          return maxlen;
        }
    }
}


bool   StreamSource_HTTP::MoreDataPending() const
{
  // TODO
  if (fp) {
    return true;
  } else {
    return false;
  }

}


uint64 StreamSource_HTTP::AskStreamLength() const
{
  // TODO
  return 0;
}


uint64 StreamSource_HTTP::AskCurrentPosition() const
{
  // TODO
  return 0;
}


uint64 StreamSource_HTTP::SetCurrentPosition(uint64 pos)
{
  // TODO
  return pos;
}


int StreamSource_HTTP::getbytedirect(void)
{
  int c;

  if((c=getc(fp))<0)
  {
    cerr << "HTTP_ERROR_FILEREADFAIL\n";
    return -1;
  }

  return c;
}

bool StreamSource_HTTP::_readbuffer(char *buffer,int size)
{
  if(fread(buffer,size,1,fp)!=1)
  {
    cerr << "HTTP_ERROR_FILEREADFAIL\n";
    return false;
  }
  return true;
}

bool StreamSource_HTTP::eof(void)
{
  return feof(fp);
};

int StreamSource_HTTP::getblock(char *buffer,int size)
{

  int l=0;
  
  if (fp) {
    l=fread(buffer,1,size,fp);
  
    if ( l==0 ) {
      cerr << "HTTP_ERROR_FILEREADFAIL(getblock)\n";
      if (fp) fclose(fp);
      fp = 0;
    }
  }
    return l;
}

int StreamSource_HTTP::getsize(void)
{
  return 0;
}

void StreamSource_HTTP::setposition(int)
{
}

int  StreamSource_HTTP::getposition(void)
{
  return 0;
}


