/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: tunermodule.cpp                                            *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "audio.h"
#include "cmdblock.h"
#include "cmdutil.h"
#include "errorbase.h"
#include "filetool.h"
#include "sharedclient.h"
#include "srctuner.h"
#include "state.h"

#include <anlg_video.h>


/*
**
** Local Constant Definitions
**
*/
#define TIMEOUT 1



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char *plugin_name = "181 Channel Analog Tuner";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "tuner";



/*
**
**
**
*/
static int loadFrame(void)
{
  int tick;

  // make sure cmd_block is idle
  if(cmd_block->status != WMSTATUS_IDLE)
    return(ERRORBASE_TUNER + 10);

  // issue the stop command
  cmd_block->command = WMCMD_VIEWER_FRAME_LOAD;
  strcpy(cmd_block->parm1,"Dialog");
  strcpy(cmd_block->parm2,"dialogtuner.html");
  cmd_block->status = WMSTATUS_START;

  // wait for the command to finish
  tick = time(NULL);
  while(1)
  {
    if(cmd_block->status == WMSTATUS_DONE)
      break;
    usleep(1000);
    if((tick + TIMEOUT) <= time(NULL))
    {
      cmd_block->command = WMCMD_NONE;
      cmd_block->status = WMSTATUS_IDLE;
      //      return(ERRORBASE_TUNER + 11);
      return(0);
    }
  }
  cmd_block->command = WMCMD_NONE;
  cmd_block->status = WMSTATUS_IDLE;
  
  return(0);
}



/*
**
**
**
*/
static int refreshFrame(void)
{
  int tick;

  // make sure cmd_block is idle
  if(cmd_block->status != WMSTATUS_IDLE)
    return(ERRORBASE_TUNER + 10);

  // issue the stop command
  cmd_block->command = WMCMD_VIEWER_OBJECT_REFRESH;
  strcpy(cmd_block->parm1,"Dialog");
  cmd_block->status = WMSTATUS_START;

  // wait for the command to finish
  tick = time(NULL);
  while(1)
  {
    if(cmd_block->status == WMSTATUS_DONE)
      break;
    usleep(1000);
    if((tick + TIMEOUT) <= time(NULL))
    {
      cmd_block->command = WMCMD_NONE;
      cmd_block->status = WMSTATUS_IDLE;
      //      return(ERRORBASE_TUNER + 11);
      return(0);
    }
  }
  cmd_block->command = WMCMD_NONE;
  cmd_block->status = WMSTATUS_IDLE;
  
  return(0);
}



/*
**
**
**
*/
static int updateChannelDisplay(void)
{
  char buf1[256],buf2[256];

  int ch,hundreds,ones,tens;

  // convert the channel number into hundreds, tens, and ones
  hundreds = tens = ones = 0;
  bzero(buf1,256);
  bzero(buf2,256);
	
   printf("First print %s %s \n", buf1, buf2);

  ch = state_channel;
  while(1)
  {
    if(ch >= 100)
    {
      ch -= 100;
      ++hundreds;
    }
    else 	
    if(ch >= 10)
    {
      ch -= 10;
      ++tens;
    }
    else
    {
      ones = ch;
      break;
    }
  }

  // copy over the hundreds image
  if(hundreds == 0)
    sprintf(buf1,"%s/images/numblank.gif",state_html_base);
  else
    sprintf(buf1,"%s/images/num%d.gif",state_html_base,hundreds);
  sprintf(buf2,"%s/images/hundreds.gif",state_html_base);
 
  copyFileToFile(buf2,buf1);
	

  // copy over the tens image
  if((hundreds == 0) && (tens == 0))
    sprintf(buf1,"%s/images/numblank.gif",state_html_base);
  else
    sprintf(buf1,"%s/images/num%d.gif",state_html_base,tens);
  sprintf(buf2,"%s/images/tens.gif",state_html_base);

  copyFileToFile(buf2,buf1);


  // copy over the ones image
  sprintf(buf1,"%s/images/num%d.gif",state_html_base,ones);
  sprintf(buf2,"%s/images/ones.gif",state_html_base);

  copyFileToFile(buf2,buf1);

  // exit with no errors
  return(0);
}


/*
**
** Command: source
**
*/
static int sourceFunc(int argc, char *argv[])
{
	int					tunerrc = 0;								// Assume success

	// TODO:
	// 	This function will soon be deprecated and replaced by NSC Middle API calls
  if(VAPISrcTunerSelect() != 0)
    return(ERRORBASE_TUNER + 0);
	
	VMW_AVS_VideoInit();
	tunerrc = VIDEO_INPUT_TUNER;
	VMW_AVS_SetCurrentVideoInput(&tunerrc);
	VMW_OVL_EnableOverlay(0x00f000f0, 0x00f0f0f0, 0);

	// Force the tuner and the state information to jive...added 06/01/01 jmw
	VMW_AVT_SetTunerFrequency((unsigned long *) &state_channel);

	VMW_AVS_VideoClose();
	strcpy(state_source,plugin_keyword);
	updateChannelDisplay();
	loadFrame();

  return(0);
}



/*
**
** Command: channelup
**
*/
static int channelupFunc(int argc, char *argv[])
{
	int					tunerrc = 0;								// assume success

  if(++state_channel > state_channel_max)
    state_channel = state_channel_min;

  	VMW_AVS_VideoInit();
	VMW_AVT_SetTunerFrequency((unsigned long *)&state_channel);
	VMW_AVS_VideoClose();
	updateChannelDisplay();
	refreshFrame();

  return(0);
}



/*
**
** Command: channeldown
**
*/
static int channeldownFunc(int argc, char *argv[])
{
	int					tunerrc = 0;								// Assume success

  if(--state_channel < state_channel_min)
    state_channel = state_channel_max;

  	VMW_AVS_VideoInit();
	VMW_AVT_SetTunerFrequency((unsigned long *)&state_channel);
	VMW_AVS_VideoClose();
	updateChannelDisplay();
	refreshFrame();

  return(tunerrc);
}



/*
**
** Command: channel n
**
*/
static int channelFunc(int argc, char *argv[])
{
  int					chnum,										// Channel number
						tunerrc = 0;								// Assume success
  chnum = atoi(argv[0]);
  if((chnum < state_channel_min) || (chnum > state_channel_max))
    return(ERRORBASE_TUNER + 7);
  state_channel = chnum;

  	VMW_AVS_VideoInit();
	VMW_AVT_SetTunerFrequency((unsigned long *)&state_channel);
	VMW_AVS_VideoClose();
	updateChannelDisplay();
	refreshFrame();

  return(0);
}



/*
**
** Command: info
**
*/
static int infoFunc(int argc, char *argv[])
{
  printf("Tuner: channel %d\n",state_channel);
  return(0);
}

/*******************************************************************************\
**
**	Function:	int volumeupFunc()
**	Desc:		Increments the volume on this audio channel by 1 unit
**	Accepts:	int argc = Number of args
**				char *argv[] = Argument vector
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumeupFunc(int argc, char *argv[])
{
	SetAudio(AV_CHAN_TUNER, 0, 1);
	return (0);
} /* end of volumeupFunc() */

/*******************************************************************************\
**
**	Function:	int volumedownFunc()
**	Desc:		Decrements the volume on this audio channel by 1 unit
**	Accepts:	int argc = Number of args
**				char *argv[] = Argument vector
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumedownFunc(int argc, char *argv[])
{
	SetAudio(AV_CHAN_TUNER, 0, -1);
	return (0);
} /* end of volumedownFunc() */

/*******************************************************************************\
**
**	Function:	int volumemuteFunc()
**	Desc:		Mutes the volume on this audio channel
**	Accepts:	int argc = Number of args
**				char *argv[] = Argument vector
**	Returns:	int; 0 on success, ERRORBASE value on error
**	
\*******************************************************************************/
static int
volumemuteFunc(int argc, char *argv[])
{
	fprintf(stderr, "Muting tuner\n");
	SetAudio(AV_CHAN_TUNER, 1, 0);
	return (0);
} /* end of volumedownFunc() */

/*******************************************************************************\
**
**	Function:	int volumeunmuteFunc()
**	Desc:		Unmutes the volume on this audio channel 
**	Accepts:	int argc = Number of args
**				char *argv[] = Argument vector
**	Returns:	int; 0 on success, ERRORBASE value on error
**
\*******************************************************************************/
static int
volumeunmuteFunc(int argc, char *argv[])
{
	fprintf(stderr, "Unmuting tuner\n");
	SetAudio(AV_CHAN_TUNER, 0, 0);
	return (0);
} /* end of volumeunmuteFunc() */


/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "source",sourceFunc,
  "channelup",channelupFunc,
  "channeldown",channeldownFunc,
  "channel",channelFunc,
  "info",infoFunc,
  "volup", volumeupFunc,
  "voldn", volumedownFunc,
  "mute", volumemuteFunc,
  "unmute", volumeunmuteFunc,
  NULL,NULL
};



/*
**
** This function will initialize the tuner module and prepare it
** for subsequent operations.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function will closeout the tuner module and shut it down.
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function will return the subsystem identification string
** for the tuner module ("tuner").
**
** The "result" parameter is a pointer to the buffer where the
** subsystem identification string will be stored.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
** This function will display the commands available for the tuner
** module.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s source\n",plugin_keyword);
  printf("   %s channelup\n",plugin_keyword);
  printf("   %s channeldown\n",plugin_keyword);
  printf("   %s channel n\n",plugin_keyword);
  printf("   %s info\n",plugin_keyword);
  printf("   %s mute\n",plugin_keyword);
  printf("   %s unmute\n",plugin_keyword);
  printf("   %s volup\n",plugin_keyword);
  printf("   %s voldn\n",plugin_keyword);
  return(0);
}



/*
**
** This function will execute the specified tuner module command.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result;

  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_TUNER + 5;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

  // return result and exit
  return(result);
}

