/***********************************************************************
*                                                                      *
* COPYRIGHT (c) 2000   Century Software   All Rights Reserved          *
*                                                                      *
* This software is the property of Century Software and shall not be   *
* reproduced or copied in whole or  used in whole or in  part as the   *
* basis for the manufacture or sale of items, nor shall such copy be   *
* sold or constitute part of a sale without written permission.        *
*                                                                      *
* RESTRICTED RIGHTS LEGEND                                             *
*                                                                      *
* Use, duplication,  or disclosure by the government is subject to     *
* restriction as set forth in paragraph (b)(3)(b) of the Rights in     *
* Technical Data and Computer Software clause in DAR 7-104.9(a).       *
*                                                                      *
* CENTURY SOFTWARE                                                     *
* 5284 SOUTH 320 WEST - SUITE C-134                                    *
* MURRAY, UTAH   84107                                                 *
*                                                                      *
************************************************************************
*                                                                      *
* FILENAME: wavplugin.cpp                                              *
*                                                                      *
***********************************************************************/



/*
**
** Imported "Include" Files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include "errorbase.h"



/*
**
** Local Constant Definitions
**
*/



/*
**
** Local Enumeration Definitions
**
*/



/*
**
** Local Structure Definitions
**
*/
typedef struct
{
  char *cmd;
  int (*func)(int argc, char *argv[]);
}CmdEntry;



/*
**
** Local Variable Declarations
**
*/
static char *plugin_name = "Audio WAV player";
static char *plugin_version = "1.0.0";
static char *plugin_keyword = "wav";



/*
**
** Command: source
**
*/
static int playFunc(int argc, char *argv[])
{
  char buf[256];

  sprintf(buf,"/usr/bin/sox %s -t ossdsp /dev/dsp",argv[0]);
  system(buf);
  return(0);
}



/*
**
** Command List
**
*/
static CmdEntry cmdlist[] =
{
  "play",playFunc,
  NULL,NULL
};



/*
**
** This function will initialize the wavaudio module and prepare it
** for subsequent operations.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginStartup(void)
{
  return(0);
}



/*
**
** This function will closeout the wav audio module and shut it down.
**
** If successful, '0' is returned. If an error occurs during function
** exeuction, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginShutdown(void)
{
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginName(char *result)
{
  strcpy(result,plugin_name);
  return(0);
}



/*
**
**
**
*/
extern "C"
int pluginVersion(char *result)
{
  strcpy(result,plugin_version);
  return(0);
}



/*
**
** This function will return the subsystem identification string
** for the wav audio module ("wav").
**
** The "result" parameter is a pointer to the buffer where the
** subsystem identification string will be stored.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginKeyword(char *result)
{
  strcpy(result,plugin_keyword);
  return(0);
}



/*
**
** This function will display the commands available for the 
** wav audio module.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginHelp(void)
{
  printf("   %s play filename\n",plugin_keyword);
  return(0);
}



/*
**
** This function will execute the specified wav audio module command.
**
** The "argc" parameter specifies the number of string parameters
** found in the argv array. The "argv" parameter is an array of
** pointers to the command and it's parameters.
**
** If successful, '0' is returned. If an error occurs during function
** execution, a non-zero value is returned that describes the error.
**
** This function is a module entrypoint.
**
*/
extern "C"
int pluginCommand(int argc, char *argv[])
{
  int count,result;

  // scan the list of commands for a match
  count = 0;
  while(1)
  {
    // check for end of list (indicates unknown command)
    if(cmdlist[count].cmd == NULL)
    {
      result = ERRORBASE_WAV + 0;
      break;
    }

    // check for command match
    if(strcmp(cmdlist[count].cmd,argv[0]) == 0)
    {
      result = (cmdlist[count].func)(argc - 1,&argv[1]);
      break;
    }

    // bump to next command
    ++count;
  }

  // return result and exit
  return(result);
}
